#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "ctest3.h"
#include "dplot.h"

#undef DOUBLEPRECISION
#ifdef DOUBLEPRECISION
#define REAL double
#define DPLOT_ADDDATA dPlot_AddData8
#else
#define REAL float
#define DPLOT_ADDDATA dPlot_AddData
#endif

float	TimeStep;
HWND	hwndDPlot;
HWND	hwndDlg;
int		DocNum;
int		DrawMethod;
int		StartTicks;
int		StopTicks;
int		Running;
int		busy;
float	StopTime=3600.0;		// Time to stop the simulation, in seconds

void RealTimeStop()
{
	int	ret;

	StopTicks = GetTickCount();
	KillTimer(hwndDlg,1);
	Running = 0;
	// Redraw the plot and re-enable menu updates
	if(DocNum) ret = dPlot_Command(DocNum, "[ViewRedraw()][MenuUpdates(1)][DeferMinMaxCheck(0)][Enable(0)]");

	// NEW in version 2.2.2.6 of DPLOTLIB.DLL: DPlot_AddData no longer cleans
	// up all DDEML data on exit. This was necessary to make real-time apps
	// workable on Vista and in .NET programs. Therefore the calling application
	// should always call dPlot_Disconnect if dPlot_AddData has been called.
	// There's no harm in calling this function if dPlot_AddData has NOT been
	// caled.
	dPlot_Disconnect();
	// Make time output invisible
	ShowWindow(GetDlgItem(hwndDlg,DLG_TIMELABEL),SW_HIDE);
	ShowWindow(GetDlgItem(hwndDlg,DLG_TIME),SW_HIDE);
	// Reset button text
	SetDlgItemText(hwndDlg,DLG_START,"&Start");
	DocNum = 0;
}

void TimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
	float	x, y1, y2;
	char	szExec[256];
	int		ret;

	if(busy) return;
	busy=1;
    x  = (dwTime - StartTicks) / 1000.f;
	if(x > StopTime)
	{
		RealTimeStop();
		return;
	}
	y1 = (REAL)sin(x/4);
	y2 = (REAL)cos(x/4);

   
/*  The most efficient method for sending data to DPlot is highly dependent
    on how many points are sent at once and how many curves are updated at
    once. For this demo it's a wash; they're roughly equivalent. The
    advantage of using DPlot_Command with an XY or XYXY command is that
    multiple curves may be updated in a single call. The advantage of using
    DPlot_AddData is that the data is sent in its native binary format, so
    there's no need for translation to character string and back to binary.
    For a large number of points this advantage becomes much more important.
    DPlot_AddData also allows you to send an arbitrarily large number of
    values; DPlot_Command is limited by the maximum command string length
    (32768 characters). */

	if(DrawMethod)
	{
		ret = DPLOT_ADDDATA(DocNum, DATA_XYXY, 1, 1, &x, &y1);
		if(ret > 0)
			ret = DPLOT_ADDDATA(DocNum, DATA_XYXY, 1, 2, &x, &y2);
	}
	else
	{
		sprintf_s(szExec,sizeof(szExec),"[SelectCurve(1)][XY(%g,%g)][SelectCurve(2)][XY(%g,%g)]",x,y1,x,y2);
		ret = dPlot_Command(DocNum,szExec);
	}
	busy = 0;
    if(ret <= 0)
		RealTimeStop();
	else
	{
		char	szTime[16];
		sprintf_s(szTime,sizeof(szTime),"%g",x);
		SetDlgItemText(hwndDlg,DLG_TIME,szTime);
	}
}

INT_PTR CALLBACK OptionsDlgProc( HWND hwnd, unsigned msg, UINT wparam, LONG lparam )
{
	char	szText[64];
	char	szExec[256];
	int		cx, cy;
    RECT	rc, rcWork;
    
	switch( msg ) {
	case WM_INITDIALOG:
		// Preserve window handle for use in other functions
		hwndDlg = hwnd;

		StopTicks = 0;

		// Initial time step = 0.1 seconds
		Running = 0;
		TimeStep = 0.1f;
		sprintf_s(szText,sizeof(szText),"%g",TimeStep);
		SetDlgItemText(hwnd,DLG_INCREMENT,szText);

		DrawMethod = 0;			// Use DPlot_Command
		CheckDlgButton(hwnd,DLG_METHOD,DrawMethod);

		// Initial plot window
		SetDlgItemText(hwnd,DLG_MSCALE,"0.,-1.5,60.,1.5");

		// Make time output invisible while simulation isn't running
		ShowWindow(GetDlgItem(hwnd,DLG_TIMELABEL),SW_HIDE);
		ShowWindow(GetDlgItem(hwnd,DLG_TIME),SW_HIDE);

		// Set dialog position to upper left corner of work area and make it
		// topmost (stay-on-top)
		SystemParametersInfo(SPI_GETWORKAREA,0,&rcWork,0);
		SetWindowPos(hwnd,HWND_TOPMOST,rcWork.left,rcWork.top,0,0,SWP_NOSIZE);

		// Start DPlot
		hwndDPlot = (HWND)dPlot_Start(0,NULL);

		// Move DPlot to upper right corner of screen, sized so that both our form
		// and DPlot are completely visible
		GetWindowRect(hwnd,&rc);
		cx = rcWork.right - rcWork.left - rc.right;
		cy = rcWork.bottom - rcWork.top;
		if(IsZoomed(hwndDPlot)) ShowWindow(hwndDPlot,SW_RESTORE);
		MoveWindow(hwndDPlot,rc.right,rcWork.top,cx,cy,1);

		// Use MessageBox to report errors
		dPlot_SetErrorMethod(2);
		
		return TRUE;
		
	case WM_COMMAND:
		switch( LOWORD( wparam ) )
		{
		case DLG_START:
			// If simulation is already running, stop it
			if(Running)
			{
				RealTimeStop();
				return TRUE;
			}
			// With a very short time step, we may end up here twice in rapid succession, 
			// and user will have a hard time stopping the demo. This gives us a
			// governor:
			if(GetTickCount()-StopTicks < 1000) return TRUE;

			// Get timestep
			GetDlgItemText(hwnd,DLG_INCREMENT,szText,sizeof(szText));
			TimeStep = (float)atof(szText);

			// Drawing method (see comments in TimerProc)
			DrawMethod = IsDlgButtonChecked(hwnd,DLG_METHOD);
    
			// Change button caption to indicate that clicking it again stops the simulation,
			// and make the time display visible
			SetDlgItemText(hwnd,DLG_START,"&Stop");
			ShowWindow(GetDlgItem(hwnd,DLG_TIMELABEL),SW_SHOW);
			ShowWindow(GetDlgItem(hwnd,DLG_TIME),SW_SHOW);

			// If DPlot is already running, bring it to the top in case user might have
			// minimized it.
			if(IsWindow(hwndDPlot)) {
				if(IsIconic(hwndDPlot))
					ShowWindow(hwndDPlot,SW_RESTORE);
				else
					BringWindowToTop(hwndDPlot);
			}

			// Create an empty document window in DPlot and get the document index, which will
			// be used for all subsequent DPlot_Command command strings.
			DocNum = dPlot_Command(0, "[FileNew()]");

			// Instruct DPLOT to allocate space for 2 curves, 1 hour's
			// worth of data:
			sprintf_s(szExec,sizeof(szExec),"[FileArrays(2,%d)]",(int)(StopTime/TimeStep-0.5)+1);
			dPlot_Command(DocNum,szExec);

			// Use thick (.04") lines to push DPlot's performance limits a bit, and
			// set a few other plot parameters
			dPlot_Command(DocNum,"[LineWidth(1,40)][LineWidth(2,40)]"
				"[LineType(1,1)][LineType(2,1)]"
				"[Color(1,255,0,0)][Color(2,0,0,255)]");

			// Get plot window
			GetDlgItemText(hwnd,DLG_MSCALE,szText,sizeof(szText));

			// Set extents and tell DPlot not to automatically recalculate mins, maxes,
			// means, etc. to help with the speed (mostly relevant for many thousands
			// of points.) Be sure to use DeferMinMaxCheck(1) when finished, or those
			// values won't be recalculated until the user does something to alter the
			// data in DPlot.
 			sprintf_s(szExec,sizeof(szExec),"[ManualScale(%s)][AutoSize()][DeferMinMaxCheck(1)]",szText);
// Test bubble plot:
// 			sprintf_s(szExec,sizeof(szExec),"[AutoSize()][GeneralOptions(32,0)][Bubbles(1,288,2,100,250,0xFF0000,0x0000FF,-1,1,3)]");
			dPlot_Command(DocNum,szExec);

			// Set labels
			dPlot_Command(DocNum,"[XAxisLabel(\"Time (seconds)\")]"
  				"[LegendParams(0.5,10,1,1,.05,.05)]"
  				"[FontPoints(1,8)][FontPoints(2,12)][FontPoints(4,12)][FontPoints(6,10)]"
  				"[Legend(1,\"SIN(t/4)\")]"
  				"[Legend(2,\"COS(t/4)\")]"
				"[Caption(\"C Demo\")]");
			if(DrawMethod)
				dPlot_Command(DocNum,"[Title1(\"RealTime Demo using DPlot_AddData\")]");
			else
				dPlot_Command(DocNum,"[Title1(\"RealTime Demo using DPlot_Command\")]");

			// Tell DPlot not to update its menu (which normally would be done every
			// time [SelectCurve()] is used). This isn't strictly necessary, but
			// avoids annoying flashing from the menu and toolbar being continually
			// redrawn, and is a bit more efficient since, for the purposes of this
			// demo, the menu doesn't really need to be updated anyway. HOWEVER,
			// this setting is persistent. If you turn off menu updates then you
			// must eventually turn them back on. In the case of this example this
			// is done when the user stops the simulation and RealTimeStop is called.
			dPlot_Command(DocNum, "[MenuUpdates(0)]");

			// Get start time from API function GetTickCount
			StartTicks = GetTickCount();

			// Start the timer. See TimerProc for sending our values to DPlot
			busy = 0;
			SetTimer(hwnd,1,(int)(TimeStep*1000),(TIMERPROC)TimerProc);
			Running = 1;

			return TRUE;

		default:
			return FALSE;

		}

	case WM_CLOSE:
		if(Running) RealTimeStop();
		EndDialog(hwnd,1);
		return TRUE;
	}
	
	return FALSE;
	
} /* PlotOptionsDlgProc */

/*
 * WinMain - initialization, message loop
 */
int PASCAL WinMain( HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline,
				   int cmdshow )
{
	DocNum = 0;
	DialogBox( this_inst,"DEMO3", (HWND)NULL, OptionsDlgProc );
	return 0;
	
} /* WinMain */


