#define INCLUDE_DDEML_H
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include "ctest4.h"
#include "dplot.h"

const float PI=3.14159265358979224f;

int				DPlotWasActive;
HENHMETAFILE	hemf;
HWND			hwndFrame;
RECT			Margin;			// Print margins in 1000ths of inches
RECT			rcFrame;
float			SizeX, SizeY;
int				dxPic, dyPic;	// Difference in size between picture frame and main window, used for resizing

typedef struct tagXYZ
{
	float x;
	float y;
	float z;
} XYZ;
//==============================================================================
void GetDesiredMetafileSize(HWND hwnd)
{
	int	ScreenRes;		// Display resolution (not generally accurate, but good enough in this case) in dpi
	HDC	hdc;

	GetWindowRect(hwndFrame,&rcFrame);
	InflateRect(&rcFrame,-3,-3);
	MapWindowPoints(NULL,hwnd,(LPPOINT)&rcFrame,2);
	// Size of the metafile to retrieve, in inches. The actual size specified isn't critical 
	// *unless* we want font sizes in our output to exactly match what we specify. What IS
	// fairly important is that the aspect ratio matches the aspect ratio of the output.

	hdc = GetDC((HWND)NULL);
	ScreenRes = GetDeviceCaps(hdc,LOGPIXELSX);
	ReleaseDC((HWND)NULL,hdc);
	if(rcFrame.right-rcFrame.left >= rcFrame.bottom-rcFrame.top)
	{
		SizeX = (float)(rcFrame.right-rcFrame.left+1)/(float)ScreenRes;
		SizeX = max(3.0f,min(SizeX,12.9f));
		/*  12.9" is about the absolute max. Metafile resolution is 2540 dpi, so 12.9" ~ 32K. Some
			versions of Windows will choke on dimensions greater than 32K.
			At the other extreme, DPlot forces both dimensions to be at least 3" in an effort to 
			prevent large fonts from overwhelming the plot. */
		SizeY = (SizeX * (float)(rcFrame.bottom-rcFrame.top+1))/(float)(rcFrame.right-rcFrame.left+1);
	}
	else
	{
		SizeY = (float)(rcFrame.bottom-rcFrame.top+1)/(float)ScreenRes;
		SizeY = max(3.0f,min(SizeY,12.9f));
		SizeX = (SizeY * (float)(rcFrame.right-rcFrame.left+1))/(float)(rcFrame.bottom-rcFrame.top+1);
	}
	/*  Make sure both width and height are at least 3" (because DPlot will FORCE them to 3" if they
		aren't. One obvious problem: if the picture frame aspect ratio is > 4.3:1 or less than 1:4.3 then
		the opposite dimension will be > 12.9", which we shouldn't allow, either. So in that case the 
		picture will end up being distorted. */
	if(SizeX < 3.0f)
	{
		SizeX = 3.0f;
		SizeY = (SizeX * (float)(rcFrame.bottom-rcFrame.top+1))/(float)(rcFrame.right-rcFrame.left+1);
		SizeY = min(SizeY,12.9f);
	}
	if(SizeY < 3.0f)
	{
		SizeY = 3.0f;
		SizeX = (SizeY * (float)(rcFrame.right-rcFrame.left+1))/(float)(rcFrame.bottom-rcFrame.top+1);
		SizeX = min(SizeX,12.9f);
	}
}
//==============================================================================
char *va(char *format, ...)
{
	va_list	argptr;
	static char	string[1024];
	va_start (argptr, format);
	vsprintf (string, format,argptr);
	va_end (argptr);
	return string;	
}
//==============================================================================
BOOL CALLBACK PlotOptionsDlgProc( HWND hwnd, unsigned msg,
								 UINT wparam, LONG lparam )
{
#ifdef __LCC__
	HDC			hDC;
#endif
	char		szExec[512];
	char		szText[256];
	float		A;
	float		StartX;
	float		EndX;
	float		dX, dY;
	float		x[2],xx,xTick;
	float		extents[4];
	float		*y;
	float		*z, *pz;
	float		XP, YP;
	int			DocNum;
	DPLOT		DPlot;
	DWORD		i, j;
	UINT		NP;
	UINT		NX, NY;
	XYZ 		*Node;
	switch( msg ) {
	case WM_INITDIALOG:
	{
		RECT	rcMain;

		A      = 0.5;
		StartX = 0.;
		EndX   = 4.;
		NP     = 1001;				// Number of points to plot
		sprintf(szText, "%.5g", A);
		SetDlgItemText(hwnd,DLG_A,szText);
		sprintf(szText, "%.5g", StartX);
		SetDlgItemText(hwnd,DLG_STARTX,szText);
		sprintf(szText, "%.5g", EndX);
		SetDlgItemText(hwnd,DLG_ENDX,szText);
		sprintf(szText, "%d", NP);
		SetDlgItemText(hwnd,DLG_NUMPOINTS,szText);
		SetDlgItemText(hwnd,DLG_NUM3DRAND,"100");
		SetDlgItemText(hwnd,DLG_NX,"50");
		SetDlgItemText(hwnd,DLG_NY,"50");

		// Default printer margins:
		Margin.left = 250;
		Margin.top = 250;
		Margin.right = 250;
		Margin.bottom = 250;

		// Picture frame for metafile
		hwndFrame = GetDlgItem(hwnd,DLG_PICFRAME);
		GetDesiredMetafileSize(hwnd);

		// Start up DPlot (or DPlot Jr). This isn't strictly necessary since all of the 
		// DPlot_Plot* functions will start DPlot themselves, but plots will be produced 
		// much faster if we don't have to restart DPlot every time. We start it hidden, 
		// and keep track of whether it was this program that opened DPlot or not with
		// "DPlotWasActive". If that value is 0, then we'll clean up after ourselves when
		// we quit by calling DPlot_Stop.
		dPlot_Start(1,&DPlotWasActive);

		// Use MessageBox for error messages in DPLOTLIB
		dPlot_SetErrorMethod(2);

		// Resizing stuff
		GetWindowRect(hwnd,&rcMain);
		GetWindowRect(hwndFrame,&rcFrame);
		dxPic = (rcMain.right-rcMain.left) - (rcFrame.right-rcFrame.left);
		dyPic = (rcMain.bottom-rcMain.top) - (rcFrame.bottom-rcFrame.top);
		return TRUE;
	}

	case WM_SIZE:
	{
		int	cx, cy;
		RECT rcMain;
		/*  Resize th picture frame along with the main window, and redraw
			the picture. We don't bother regenerating the metafile with a 
			new size, it will simply be stretched. */
		GetWindowRect(hwnd,&rcMain);
		cx = (rcMain.right-rcMain.left) - dxPic;
		cy = (rcMain.bottom-rcMain.top) - dyPic;
		SetWindowPos(hwndFrame,(HWND)NULL,0,0,cx,cy,SWP_NOMOVE | SWP_NOZORDER);
 		InvalidateRect(hwndFrame,(RECT *)NULL,TRUE);
		return FALSE;
	}

	case WM_CLOSE:
		if(!DPlotWasActive) dPlot_Stop();
		EndDialog(hwnd,TRUE);
		return TRUE;

	case WM_COMMAND:
		switch( LOWORD( wparam ) ) {

		case DLG_PLOTXY:
			GetDlgItemText(hwnd,DLG_A,szText,sizeof(szText));
			A = (float)atof(szText);
			GetDlgItemText(hwnd,DLG_STARTX,szText,sizeof(szText));
			StartX = (float)atof(szText);
			GetDlgItemText(hwnd,DLG_ENDX,szText,sizeof(szText));
			EndX = (float)atof(szText);
			GetDlgItemText(hwnd,DLG_NUMPOINTS,szText,sizeof(szText));
			NP = atoi(szText);
			
			if( NP < 2 ) {
				MessageBox(hwnd,"The number of points must be greater than 1.","Demo",MB_OK);
				return TRUE;
			}

			/*  Disable all command buttons to prevent multiple simultaneous calls to
				DPLOTLIB functions. Be sure we can't return without re-enabling those
				buttons. For more information see the "Timing Issues" topic in the
				DPLOTLIB Help file.
			*/
			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),FALSE);

			dX = (EndX-StartX)/(NP-1);
			x[0] = dX;
			x[1] = StartX;
			y = (float *)calloc( NP*2, sizeof(float));
			for(i=0; i<NP; i++)
			{
				xx      = StartX + i*dX;
				y[i]    = (float)sin(A*PI*xx);
				y[i+NP] = (float)cos(A*PI*xx);
			}
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_DXY;
			DPlot.MaxCurves		= 2;	// Must be >= number of curves we plot
			DPlot.MaxPoints		= NP;	// Anything >= NP will do
			DPlot.NumCurves		= 2;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			DPlot.LegendX		= 0.05f;
			DPlot.LegendY		= 0.05f;
			DPlot.NP[0]			= NP;
			DPlot.NP[1]			= NP;
			DPlot.LineType[0]	= LINESTYLE_SOLID;
			DPlot.LineType[1]	= LINESTYLE_LONGDASH;
			if(A == 1.0)
			{
				strcpy(DPlot.Legend[1],"sin({\\sp}x)");	// {\s is DPlot-speak for "use symbol font"
				strcpy(DPlot.Legend[2],"cos({\\sp}x)");
			}
			else
			{
				sprintf(DPlot.Legend[1],"sin(%g{\\sp}x)",A);
				sprintf(DPlot.Legend[2],"cos(%g{\\sp}x)",A);
			}
			strcpy(DPlot.Title[0],"DPLOT_GetEnhMetaFile Demo");
 			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			// Take a stab at a reasonable tick mark interval on the X axis
			xTick = 0.25;
			while((EndX-StartX)/xTick > 12)
				xTick *= 2;
  			sprintf(szExec,"[ManualScale(%g,-1.25,%g,1.25)][TickInterval(1,%g,0.25)]"
				"[Caption(\"CTEST4 Example 1\")]"
				"[Color(1,255,0,0)][Color(2,0,0,255)]"
				"[LineWidth(-2,20)][LineWidth(-1,20)][LineWidth(0,10)]"
				"[TextFont(1, 8,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(2,12,700,0,0,0,0,\"Arial\")]"
				"[TextFont(4,10,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(5,10,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(6,10,  0,0,0,0,0,\"Arial\")]"
				"[ClearEditFlag()]",StartX,EndX,xTick);
			if(hemf) 
			{
				DeleteEnhMetaFile(hemf);
				hemf = (HENHMETAFILE)NULL;
			}

			DocNum = dPlot_Plot(&DPlot,x,y,szExec);
			// Desired metafile size is recalculated in case user resized window
			GetDesiredMetafileSize(hwnd);

			hemf = (HENHMETAFILE)dPlot_GetEnhMetaFile(DocNum,SizeX,SizeY);
			dPlot_Command(DocNum,"[FileClose()]");

/*
  STM_SETIMAGE doesn't seem to work with IMAGE_ENHMETAFILE and the LCC compiler.
  (IMAGE_BITMAP works fine - see the ctest2 demo.) The hemf IS valid - you
  can see that by copying or printing it. 
  About the only thing to do, if you need to do it, is use PlayEnhMetafile and 
  update the window on resize. That opens a can of worms for resizing and
  covering/uncovering this app's window, though, and isn't handled by this
  program.
*/
#ifdef __LCC__
			hDC = GetDC(hwnd);
			FillRect(hDC,&rcFrame,GetStockObject(WHITE_BRUSH));
			PlayEnhMetaFile(hDC,hemf,&rcFrame);
			ReleaseDC(hwnd,hDC);
#else
			SendMessage(hwndFrame,STM_SETIMAGE,IMAGE_ENHMETAFILE,(LPARAM)hemf);
#endif
			free(y);
			EnableWindow(GetDlgItem(hwnd,DLG_COPY),(hemf ? TRUE : FALSE));
			EnableWindow(GetDlgItem(hwnd,DLG_PRINT),(hemf ? TRUE : FALSE));

			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),TRUE);

			return TRUE;

		case DLG_PLOT3DRAND:
			// Plot random 3D points, generating a contour plot
			GetDlgItemText(hwnd,DLG_NUM3DRAND,szText,sizeof(szText));
			NP = atoi(szText);
			if( NP < 3 ) {
				MessageBox(hwnd,"The number of points must be greater than 2.","Demo",MB_OK);
				return TRUE;
			}
			// Generate the data. - NP randomly placed points in the range -PI < X < PI, -PI < Y < PI, Z=cos(X)*sin(Y)
			if( NULL == (Node = (XYZ *)calloc( NP, sizeof(XYZ)))) {
				MessageBox(hwnd,"Memory allocation failure.","Demo",
					MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}

			/*  Disable all command buttons to prevent multiple simultaneous calls to
				DPLOTLIB functions. Be sure we can't return without re-enabling those
				buttons. For more information see the "Timing Issues" topic in the
				DPLOTLIB Help file.
			*/
			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),FALSE);

			for( j=0; j < NP; j++ ) {
  				Node[j].x = -PI + 2*PI * (float)rand()/(float)RAND_MAX;
  				Node[j].y = -PI + 2*PI * (float)rand()/(float)RAND_MAX;
				Node[j].z = (float)(cos(Node[j].x) * sin(Node[j].y));
			}

			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version	 = DPLOT_DDE_VERSION;
			DPlot.hwnd		 = (DWORD)hwnd;
			DPlot.DataFormat  = DATA_3DR;
			DPlot.MaxCurves	 = 1;				   // ALWAYS 1 for 3DR
			DPlot.MaxPoints	 = NP;
			DPlot.NumCurves	 = 1;				   // Ditto
			DPlot.Scale		 = SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"DPLOT_GetEnhMetaFile Demo");
			strcpy(DPlot.XAxis,"X");
			strcpy(DPlot.YAxis,"Y");
			sprintf(szExec,"[ContourLegend(0)][Contour3D(0)][ContourMethod(1)][ContourGrid(0)]"
				"[ContourLevels(21,-1,1)]"
				"[ContourColorScheme(1)]"
				"[Caption(\"CTEST4 Example 2\")]"
				"[TextFont(1, 8,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(2,12,700,0,0,0,0,\"Arial\")]"
				"[TextFont(4,10,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(5,10,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(6,10,  0,0,0,0,0,\"Arial\")]");
			
			if(hemf) 
			{
				DeleteEnhMetaFile(hemf);
				hemf = (HENHMETAFILE)NULL;
			}
			DocNum = dPlot_Plot(&DPlot,NULL,(float *)Node,szExec);
			// Desired metafile size is recalculated in case user resized window
			GetDesiredMetafileSize(hwnd);

			hemf = (HENHMETAFILE)dPlot_GetEnhMetaFile(DocNum,SizeX,SizeY);
			dPlot_Command(DocNum,"[FileClose()]");

#ifdef __LCC__
			hDC = GetDC(hwnd);
			FillRect(hDC,&rcFrame,GetStockObject(WHITE_BRUSH));
			PlayEnhMetaFile(hDC,hemf,&rcFrame);
			ReleaseDC(hwnd,hDC);
#else
			SendMessage(hwndFrame,STM_SETIMAGE,IMAGE_ENHMETAFILE,(LPARAM)hemf);
#endif
			free(Node);
			EnableWindow(GetDlgItem(hwnd,DLG_COPY),(hemf ? TRUE : FALSE));
			EnableWindow(GetDlgItem(hwnd,DLG_PRINT),(hemf ? TRUE : FALSE));

			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),TRUE);

			return TRUE;

		case DLG_PLOT3DGRID:
			// Plot 3D points on a rectangular grid, generating a contour plot
			GetDlgItemText(hwnd,DLG_NX,szText,sizeof(szText));
			NX = atoi(szText);
			GetDlgItemText(hwnd,DLG_NY,szText,sizeof(szText));
			NY = atoi(szText);

			if( NX < 1 ) {
				MessageBox(hwnd,"NX must be greater than 0.","Demo",MB_OK);
				return TRUE;
			}
			if( NY < 1 ) {
				MessageBox(hwnd,"NY must be greater than 0.","Demo",MB_OK);
				return TRUE;
			}

			// Generate the data in the range -PI < X < PI, -PI < Y < PI, Z=cos(X)*sin(Y)
			if( NULL == (z = (float *)calloc( (NX+1)*(NY+1), sizeof(float)))) {
				MessageBox(hwnd,"Memory allocation failure.","Demo",
				   MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}

			/*  Disable all command buttons to prevent multiple simultaneous calls to
				DPLOTLIB functions. Be sure we can't return without re-enabling those
				buttons. For more information see the "Timing Issues" topic in the
				DPLOTLIB Help file.
			*/
			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),FALSE);

			extents[0] = -PI;		// Xll
			extents[1] = -PI;		// Yll
			extents[2] =  PI;		// Xur
			extents[3] =  PI;		// Yur

			dX = (extents[2]-extents[0]) / NX;
			dY = (extents[3]-extents[1]) / NY;
			pz = z;
			for(i=0; i<=NX; i++)
			{
				XP = -PI + i*dX;
				for(j=0; j<=NY; j++)
				{
					YP = -PI + j*dY;
					*pz = (float)(cos(XP) * sin(YP));
					pz++;
				}
			}
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version	 = DPLOT_DDE_VERSION;
			DPlot.hwnd		 = (DWORD)hwnd;
			DPlot.DataFormat = DATA_3D;
			DPlot.MaxCurves	 = NX;
			DPlot.MaxPoints	 = NY;
			DPlot.NumCurves	 = 1;
			DPlot.Scale		 = SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"DPLOT_GetEnhMetaFile Demo");
			strcpy(DPlot.XAxis,"X");
			strcpy(DPlot.YAxis,"Y");
			sprintf(szExec,"[ContourLegend(0)][Contour3D(0)][ContourMethod(1)][ContourGrid(0)]"
				"[ContourLevels(21,-1,1)]"
				"[Caption(\"CTEST4 Example 3\")]"
				"[TextFont(1, 8,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(2,12,700,0,0,0,0,\"Arial\")]"
				"[TextFont(4,10,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(5,10,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(6,10,  0,0,0,0,0,\"Arial\")]"
				"[ContourColorScheme(2)]"
				"[ContourCustomColors(%d,%d,%d,%d,%d,%d)]",
				0x0000FF00,0x0000CC33,0x00009966,0x00006699,0x000033CC,0x000000FF);
			if(hemf) 
			{
				DeleteEnhMetaFile(hemf);
				hemf = (HENHMETAFILE)NULL;
			}
			DocNum = dPlot_Plot(&DPlot,extents,z,szExec);
			// Desired metafile size is recalculated in case user resized window
			GetDesiredMetafileSize(hwnd);

			hemf = (HENHMETAFILE)dPlot_GetEnhMetaFile(DocNum,SizeX,SizeY);
 			dPlot_Command(DocNum,"[FileClose()]");

#ifdef __LCC__
			hDC = GetDC(hwnd);
			FillRect(hDC,&rcFrame,GetStockObject(WHITE_BRUSH));
			PlayEnhMetaFile(hDC,hemf,&rcFrame);
			ReleaseDC(hwnd,hDC);
#else
			SendMessage(hwndFrame,STM_SETIMAGE,IMAGE_ENHMETAFILE,(LPARAM)hemf);
#endif
			free(z);
			EnableWindow(GetDlgItem(hwnd,DLG_COPY),(hemf ? TRUE : FALSE));
			EnableWindow(GetDlgItem(hwnd,DLG_PRINT),(hemf ? TRUE : FALSE));

			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),TRUE);

			return TRUE;

		case DLG_EXITBUTTON:
			EndDialog( hwnd, TRUE );
			return TRUE;

		case DLG_COPY:
			if(hemf)
			{
				HENHMETAFILE	hemfCopy;
				OpenClipboard(hwnd);
				EmptyClipboard();
				hemfCopy = CopyEnhMetaFile(hemf,(LPCTSTR)NULL);
				SetClipboardData(CF_ENHMETAFILE,hemfCopy);
				CloseClipboard();
			}
			else
				MessageBeep(-1);
			return TRUE;

		case DLG_PRINT:
			if(hemf)
			{
				PAGESETUPDLG	psd;

				memset(&psd, 0, sizeof(PAGESETUPDLG));
				psd.lStructSize = sizeof(PAGESETUPDLG);
				psd.hwndOwner = hwnd;
				psd.Flags = PSD_MARGINS | PSD_INTHOUSANDTHSOFINCHES;
				CopyRect(&psd.rtMargin,&Margin);
				if( PageSetupDlg(&psd) )
				{
					DEVNAMES	*lpDevNames;
					DEVMODE		*lpDevMode;
					HDC			hDC;

					CopyRect(&Margin,&psd.rtMargin);
					lpDevNames = GlobalLock( psd.hDevNames );
					lpDevMode  = GlobalLock( psd.hDevMode );
					hDC = CreateDC( (LPTSTR)lpDevNames + lpDevNames->wDriverOffset,
									(LPTSTR)lpDevNames + lpDevNames->wDeviceOffset,
									(LPTSTR)lpDevNames + lpDevNames->wOutputOffset,
									lpDevMode);
					if( hDC )
					{
					    DOCINFO		DocInfo;
					    RECT		rect;
						int			resX, resY;
						float		MaxSizeX, MaxSizeY;
						float		ActSizeX, ActSizeY;
						float		scale;

						resX = GetDeviceCaps(hDC,LOGPIXELSX);
						resY = GetDeviceCaps(hDC,LOGPIXELSY);
						MaxSizeX = GetDeviceCaps(hDC,HORZRES) - (float)(Margin.left+Margin.right)*resX/1000;
						MaxSizeY = GetDeviceCaps(hDC,VERTRES) - (float)(Margin.top+Margin.bottom)*resY/1000;
						EnableWindow(hwnd, FALSE );
						memset(&DocInfo, 0, sizeof(DOCINFO));
						DocInfo.cbSize = sizeof(DOCINFO);
						DocInfo.lpszDocName = "DPlot_GetEnhMetaFile Demo";
						if( StartDoc(hDC, &DocInfo) )
						{
							StartPage(hDC);
							// Center on the page, with at least PRINT_MARGIN margins.
							// Compress the image if necessary, maintaining the
							// SizeX/SizeY aspect ratio
							ActSizeX = SizeX * resX;
							ActSizeY = SizeY * resY;
							if(ActSizeX > MaxSizeX || ActSizeY > MaxSizeY)
							{
								scale = min(MaxSizeX/ActSizeX, MaxSizeY/ActSizeY);
								ActSizeX *= scale;
								ActSizeY *= scale;
							}
							rect.left   = (int)(Margin.left*resX/1000 + (MaxSizeX-ActSizeX)/2);
							rect.top    = (int)(Margin.top*resY/1000 + (MaxSizeY-ActSizeY)/2);
							rect.right  = rect.left + (int)ActSizeX;
							rect.bottom = rect.top + (int)ActSizeY;

							PlayEnhMetaFile(hDC,hemf,&rect);
							EndPage(hDC);
							EndDoc(hDC);
						}
						DeleteDC(hDC);
					}
					EnableWindow(hwnd,TRUE);
					GlobalUnlock(psd.hDevMode);
					GlobalUnlock(psd.hDevNames);
					GlobalFree(psd.hDevMode);
					GlobalFree(psd.hDevNames);
				}
			}
			else
				MessageBeep(-1);
			return TRUE;

		case IDCANCEL:
			EndDialog( hwnd, TRUE );
			return TRUE;

		default:
			return FALSE;

		}

		case WM_CTLCOLORSTATIC:			// Set the background of our picture frame to white
			if((HWND)lparam == hwndFrame)
				return (BOOL)GetStockObject(WHITE_BRUSH);
			else
				return FALSE;
	}
	
	return FALSE;
	
} /* PlotOptionsDlgProc */
//==============================================================================
/*
* WinMain - initialization, message loop
*/
int PASCAL WinMain( HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline,
				   int cmdshow )
{
#ifdef __GNUC__
	HINSTANCE dll;
    if((dll = dPlot_Init()) == NULL) return 0;
#endif

	hemf = (HENHMETAFILE)NULL;
	DialogBox( this_inst,"DEMO4", NULL, PlotOptionsDlgProc );
	if(hemf) DeleteEnhMetaFile(hemf);

#ifdef __GNUC__
	FreeLibrary(dll);
#endif
	return 0;
	
} /* WinMain */


