#define INCLUDE_DDEML_H
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include "ctest2.h"
#include "dplot.h"

/* PLEASE READ: There's a bit of magic under the hood in this demo, so please read these comments before
   revising any of the source or copy/pasting the source into your own application.

   This demo gets a bitmap from DPlot of 1 of the 3 options. It also retrieves scaling information in the
   DPLOT_PLOTMETRICS structure so that we can convert mouse position to plot space and print the mouse
   location as X,Y values. If you don't care about mapping mouse movement, use DPlot_GetBitmap rather than
   DPlot_GetBitmapEx, delete the handling of the WM_MOUSEMOVE message, and you can ignore most of the rest
   of these notes.

   For X and Y scaling information, everything we need to know is returned in the DPLOT_PLOTMETRICS structure
   and we can close the plot using FileClose() after getting the bitmap using DPlot_GetBitmapEx. This is 
   true for XY plots as well as 2D plots of 3D data. However, if we also want Z then we have to get it from 
   DPlot, and the plot *must* remain open. This presents a problem if your application will be producing
   many (more than 32) plots - you'll need to eventually close those plots or you'll generate too many
   document windows in DPlot and get an error when you try to produce a new plot. It's fairly simple in this
   application - we just use FileClose() for the last document before creating a new one. FileClose() won't 
   generate an error for documents that have already been closed, so we could, for example, close the XY plot
   example immediately after getting its bitmap, and close it "again" before creating a new XY plot, w/o error.
   If for whatever reason you need multiple plots open, then you'll need to keep track of the document index
   associated with each plot, and use FileClose() on each of them sooner or later.

   Mouse movement: As mentioned previously, getting scaled X and Y values out of mouse position is handled
   entirely within this application and is very fast. So for only X,Y values we don't need to concern ourselves
   with flooding DPlot with DDE requests, since none are made. But we get Z values from DPlot, not from any
   simple structure that we retrieved from DPlot. There is no way that DDE and DPlot can keep up with the
   frequency of WM_MOUSEMOVE messages this application will get. So if we don't take precautions we'll end up
   with, for example, 20 requests for Z value sent to DPlot before the first one is returned, which gets ugly
   very quickly. If you use [SetErrorMethod(2)] w/o putting on the brakes then you'll be flooded with MessageBox's.
   We handle the problem in this application with the global int "busy". If we're busy, WM_MOUSEMOVE is 
   ignored. We also check to ensure that the mouse has actually moved since the last WM_MOUSEMOVE by storing
   the last mouse location.

   X and Y from mouse movement for 3D views: Cannot be done. Think about it and you'll understand why. DPlot will
   not generate an error message if you call DPlot_GetBitmapEx with a 3D plot viewed in 3D, but the returned
   DPLOT_PLOTMETRICS structure will be completely useless.
*/

const double PI=3.14159265358979224f;

int		cxBitmap, cyBitmap;		// Dimensions of the bitmap we'll retrieve from DPlot
int		DPlotWasActive;			// Indicates whether DPlot was already active when this application attempted to start it
HBITMAP	hbmp;
RECT	rcFrame;
DPLOT_PLOTMETRICS DPM;
int		Is3D;					// Flag indicating whether we're working with a surface plot
int		Is3DView=0;
int		Lighting=2;				// Phong shading is our default lighting for 3D views
int		DocNum;					// Index of the last-created plot document, 1-32
int		busy=0;					// Set to non-zero while responding to mouse movement, getting X,Y, and possibly Z values
								//   from our plot.
int		dxPic, dyPic;			// Difference in size between main window and picture frame, used for resizing
int		xOffset=0;				// Distance from top left corner of frame to top left corner of
int		yOffset=0;				//   bitmap. Will always be 0 initially, since we size the bitmap to fill
								//   the frame. Will be non-zero after resizing (frame has SS_CENTERIMAGE
								//   style, so the bitmap won't be stretched to fill the frame. *Without*
								//   SS_CENTERIMAGE the bitmap will be stretched, and we'll have to either
								//   generate a new bitmap or scale the DPM values for mouse movement code
								//   to work correctly.)
int		t1, t2;
HWND	hwndFrame;
char	szTime[32];
POINT	ptPrev;
typedef struct tagXYZ
{
	double x;
	double y;
	double z;
} XYZ;

void SetBitmapToFrame()
{
	DPM.size = sizeof(DPM);
	hbmp = (HBITMAP)dPlot_GetBitmapEx(DocNum, cxBitmap, cyBitmap, &DPM);
	if(hbmp)
	{
		xOffset = yOffset = 0;
		SendMessage(hwndFrame,STM_SETIMAGE,IMAGE_BITMAP,(LPARAM)hbmp);
	}
}

BOOL CALLBACK PlotOptionsDlgProc( HWND hwnd, unsigned msg,
								 UINT wparam, LONG lparam )
{
	char		szExec[1024];
	char		szText[256];
	double		A;
	double		StartX;
	double		EndX;
	double		dX, dY;
	double		x[2],xx,xTick;
	double		extents[4];
	double		*y;
	double		*z, *pz;
	double		XP, YP;
	DPLOT		DPlot;
	DWORD		i, j;
	UINT		NP;
	UINT		NX, NY;
	XYZ 		*Node;
	
	switch( msg ) {
	case WM_INITDIALOG:
	{
		RECT	rcMain, rcPic;

		A      = 0.5;
		StartX = 0.;
		EndX   = 4.;
		NP     = 1001;				// Number of points to plot
		sprintf(szText, "%.5g", A);
		SetDlgItemText(hwnd,DLG_A,szText);
		sprintf(szText, "%.5g", StartX);
		SetDlgItemText(hwnd,DLG_STARTX,szText);
		sprintf(szText, "%.5g", EndX);
		SetDlgItemText(hwnd,DLG_ENDX,szText);
		sprintf(szText, "%d", NP);
		SetDlgItemText(hwnd,DLG_NUMPOINTS,szText);
		CheckDlgButton(hwnd,DLG_3D,Is3DView);
		CheckRadioButton(hwnd,DLG_LIGHT_NONE,DLG_LIGHT_PHONG,DLG_LIGHT_NONE+Lighting);
		if(!Is3DView)
		{
			EnableWindow(GetDlgItem(hwnd,DLG_LIGHT_GROUP  ),0);
			EnableWindow(GetDlgItem(hwnd,DLG_LIGHT_NONE   ),0);
			EnableWindow(GetDlgItem(hwnd,DLG_LIGHT_GOURAUD),0);
			EnableWindow(GetDlgItem(hwnd,DLG_LIGHT_PHONG  ),0);
		}
		SetDlgItemText(hwnd,DLG_NUM3DRAND,"100");
		SetDlgItemText(hwnd,DLG_NX,"50");
		SetDlgItemText(hwnd,DLG_NY,"50");
		SetDlgItemText(hwnd,DLG_COORDS,"");

		// Desired size of bitmap retrieved from DPlot:
		hwndFrame = GetDlgItem(hwnd,DLG_BMPFRAME);
		GetWindowRect(hwndFrame,&rcFrame);
		InflateRect(&rcFrame,-3,-3);
		MapWindowPoints(NULL,hwnd,(LPPOINT)&rcFrame,2);
		cxBitmap = rcFrame.right-rcFrame.left+1;
		cyBitmap = rcFrame.bottom-rcFrame.top+1;

		// Start up DPlot (or DPlot Jr). This isn't strictly necessary
		// since all of the DPlot_Plot* functions will start DPlot themselves,
		// but plots will be produced much faster if we don't have to
		// restart DPlot every time. We start it minimized, and 
		// keep track of whether it was this program that opened DPlot or
		// not with "DPlotWasActive". If that value is 0, then we'll 
		// clean up after ourselves when we quit by calling DPlot_Stop.
 		dPlot_Start(1,&DPlotWasActive);

        // This demo uses an updated DPLOT_PLOTMETRICSEX structure, used to translate
		// mouse movement to data space. This change requires version 2.1.4 or later
		// of DPlot or DPlot Jr. If you don't do anything with mouse movement or if
		// you use the old version of the DPLOT_PLOTMETRICSEX structure, you can 
		// remove these lines:
		if(dPlot_MinVersion(2, 1, 4, 0) == 0)
		{
			MessageBox((HWND)NULL,"This demo requires DPlot or DPlot Jr 2.1.4.0 or later.", "C DPLOTLIB Demo",0);
			EndDialog(hwnd,0);
			return TRUE;
		}

		// Use MessageBox for error messages in DPLOTLIB
		dPlot_SetErrorMethod(2);

		// Resizing stuff
		GetWindowRect(hwnd,&rcMain);
		GetWindowRect(hwndFrame,&rcPic);
		dxPic = (rcMain.right-rcMain.left) - (rcPic.right-rcPic.left);
		dyPic = (rcMain.bottom-rcMain.top) - (rcPic.bottom-rcPic.top);

		return TRUE;
	}

	case WM_SIZE:
	{
		int	cx, cy;
		RECT rcMain;

		GetWindowRect(hwnd,&rcMain);
		cx = (rcMain.right-rcMain.left) - dxPic;
		cy = (rcMain.bottom-rcMain.top) - dyPic;
		SetWindowPos(hwndFrame,(HWND)NULL,0,0,cx,cy,SWP_NOMOVE | SWP_NOZORDER);
		GetWindowRect(hwndFrame,&rcFrame);
		InflateRect(&rcFrame,-3,-3);
		MapWindowPoints(NULL,hwnd,(LPPOINT)&rcFrame,2);
		xOffset = (rcFrame.right-rcFrame.left+1 - cxBitmap)/2;
		yOffset = (rcFrame.bottom-rcFrame.top+1 - cyBitmap)/2;
		cxBitmap = rcFrame.right-rcFrame.left+1;
		cyBitmap = rcFrame.bottom-rcFrame.top+1;
  		InvalidateRect(hwndFrame,(RECT *)NULL,TRUE);
		return FALSE;
	}

	case WM_DESTROY:
		if(DocNum) dPlot_Command(DocNum, "[FileClose()]");
		if(!DPlotWasActive) dPlot_Stop();
		return FALSE;

	case WM_CLOSE:
		EndDialog(hwnd,TRUE);
		return TRUE;

	case WM_MOUSEMOVE:
		if(hbmp && !busy)
		{
			POINT pt;
			pt.x=LOWORD(lparam);
			pt.y=HIWORD(lparam);
			if((pt.x == ptPrev.x) && (pt.y == ptPrev.y)) return FALSE;
			if(Is3D && Is3DView)
			{
				SetDlgItemText(hwnd,DLG_COORDS,"");
				return FALSE;
			}
			ptPrev = pt;
			busy=1;
			if(PtInRect(&rcFrame,pt))
			{
				double	worldX;
				double	worldY;
				double	worldZ;
				int		plotX;
				int		plotY;

				plotX = pt.x - rcFrame.left - xOffset;
				plotY = pt.y - rcFrame.top - yOffset;
  				if(plotX < DPM.hll || plotX > DPM.hur || plotY < DPM.vur || plotY > DPM.vll)
   					SetDlgItemText(hwnd,DLG_COORDS,"");
  				else
				{
					char	text[128];
					// This is only valid for linear scaling (which all of these examples use). For other
					// scaling types see the DPLOTLIB documentation
					worldX = DPM.xlo + (plotX - DPM.hll) * (DPM.xhi - DPM.xlo) / (DPM.hur - DPM.hll);
					worldY = DPM.ylo + (plotY - DPM.vll) * (DPM.yhi - DPM.ylo) / (DPM.vur - DPM.vll);
 					sprintf(text,"X= %.4g\nY= %.4g",worldX,worldY);
					if(Is3D)
					{
						int ret=dPlot_ZFromXY(DocNum, worldX, worldY, &worldZ);
						if(ret==1)
						{
							char	*p;
							p = text;
							p += strlen(text);
							sprintf(p,"\nZ= %.4g",worldZ);
						}
					}
					SetDlgItemText(hwnd,DLG_COORDS,text);
				}
			}
			busy=0;
		}
		return FALSE;

	case WM_COMMAND:
		switch( LOWORD( wparam ) ) {

		case DLG_PLOTXY:
			t1 = GetTickCount();
			GetDlgItemText(hwnd,DLG_A,szText,sizeof(szText));
			A = (double)atof(szText);
			GetDlgItemText(hwnd,DLG_STARTX,szText,sizeof(szText));
			StartX = (double)atof(szText);
			GetDlgItemText(hwnd,DLG_ENDX,szText,sizeof(szText));
			EndX = (double)atof(szText);
			GetDlgItemText(hwnd,DLG_NUMPOINTS,szText,sizeof(szText));
			NP = atoi(szText);
			
			if( NP < 2 ) {
				MessageBox(hwnd,"The number of points must be greater than 1.","Demo",MB_OK);
				return TRUE;
			}

			/*  Disable all command buttons to prevent multiple simultaneous calls to
				DPLOTLIB functions. Be sure we can't return without re-enabling those
				buttons. For more information see the "Timing Issues" topic in the
				DPLOTLIB Help file.
			*/
			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),FALSE);
			
			dX = (EndX-StartX)/(NP-1);
			StartX = 0.0;
			x[0] = dX;
			x[1] = StartX;
			y = (double *)calloc( NP*2, sizeof(double));
			for(i=0; i<NP; i++)
			{
				xx      = StartX + i*dX;
				y[i]    = (double)sin(A*PI*xx);
				y[i+NP] = (double)cos(A*PI*xx);
			}
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_DXY;
			DPlot.MaxCurves		= 2;	// Must be >= number of curves we plot
			DPlot.MaxPoints		= NP;	// Anything >= NP will do
			DPlot.NumCurves		= 2;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			DPlot.LegendX		= 0.05f;
			DPlot.LegendY		= 0.05f;
			DPlot.NP[0]			= NP;
			DPlot.NP[1]			= NP;
			DPlot.LineType[0]	= LINESTYLE_SOLID;
			DPlot.LineType[1]	= LINESTYLE_LONGDASH;
			if(A == 1.0)
			{
				strcpy(DPlot.Legend[1],"sin({\\sp}x)");	// {\s is DPlot-speak for "use symbol font"
				strcpy(DPlot.Legend[2],"cos({\\sp}x)");
			}
			else
			{
				sprintf(DPlot.Legend[1],"sin(%g{\\sp}x)",A);
				sprintf(DPlot.Legend[2],"cos(%g{\\sp}x)",A);
			}
			strcpy(DPlot.Title[0],"DPLOT_GetBitmap Demo");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			// Take a stab at a reasonable tick mark interval on the X axis
			xTick = 0.25;
			while((EndX-StartX)/xTick > 12)
				xTick *= 2;
			sprintf(szExec,"[ManualScale(%g,-1.25,%g,1.75)][TickInterval(1,%g,0.25)]"
  				"[Caption(\"DPLOTLIB XY Test\")]"
				"[GeneralOptions(0x00000020,%d)]"
				"[Color(1,255,0,0)][Color(2,0,0,255)]"
				"[TextFont(1, 8,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(2,12,700,0,0,0,0,\"Arial\")]"
				"[TextFont(4,10,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(5,10,  0,0,0,0,0,\"Arial\")]"
				"[TextFont(6,10,  0,0,0,0,0,\"Arial\")]"
				"[ClearEditFlag()]",StartX,EndX,xTick,
				(IsDlgButtonChecked(hwnd,DLG_ANTIALIAS) ? 0x00000020 : 0));

			if(hbmp)
			{
				DeleteObject(hbmp);
				hbmp = 0;
			}
			if(DocNum) dPlot_Command(DocNum, "[FileClose()]");
			DocNum = dPlot_Plot8(&DPlot,x,y,szExec);
			if(DocNum)
			{
				SetBitmapToFrame();
				Is3D = 0;
			}
			free(y);
			t2 = GetTickCount();
			sprintf(szTime,"%d",t2-t1);
			SetDlgItemText(hwnd,DLG_TIMER,szTime);

			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),TRUE);

			break;

		case DLG_PLOT3DRAND:
			t1 = GetTickCount();
			/* Plot random 3D points, generating a contour plot.*/
			GetDlgItemText(hwnd,DLG_NUM3DRAND,szText,sizeof(szText));
			NP = atoi(szText);
			if( NP < 3 ) {
				MessageBox(hwnd,"The number of points must be greater than 2.","Demo",MB_OK);
				return TRUE;
			}
			/* Generate the data in the range -PI < X < PI, -PI < Y < PI, Z=cos(X)*sin(Y) */
			if( NULL == (Node = (XYZ *)calloc( NP, sizeof(XYZ)))) {
				MessageBox(hwnd,"Memory allocation failure.","Demo",
					MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}

			/*  Disable all command buttons to prevent multiple simultaneous calls to
				DPLOTLIB functions. Be sure we can't return without re-enabling those
				buttons. For more information see the "Timing Issues" topic in the
				DPLOTLIB Help file.
			*/
			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),FALSE);
			
			for( j=0; j < NP; j++ ) {
  				Node[j].x = -PI + 2*PI * (double)rand()/(double)RAND_MAX;
  				Node[j].y = -PI + 2*PI * (double)rand()/(double)RAND_MAX;
  				Node[j].z = (double)(cos(Node[j].x) * sin(Node[j].y));
			}
			/* Initialize DPLOT structure to 0's: */
			memset(&DPlot,0,sizeof(DPlot));
			/* Fill in members of DPLOT structure */
			DPlot.Version	 = DPLOT_DDE_VERSION;
			DPlot.hwnd		 = (DWORD)hwnd;
			DPlot.DataFormat  = DATA_3DR;
			DPlot.MaxCurves	 = 1;				   // ALWAYS 1 for 3DR
			DPlot.MaxPoints	 = NP;
			DPlot.NumCurves	 = 1;				   // Ditto
			DPlot.Scale		 = SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"DPLOT_GetBitmap Demo");
			strcpy(DPlot.XAxis,"X");
			strcpy(DPlot.YAxis,"Y");

			Is3DView = IsDlgButtonChecked(hwnd,DLG_3D);
			if(Is3DView)
				sprintf(szExec,"[Caption(\"DPLOTLIB Surface Plot\")]"
				               "[Contour3D(1)][ContourView(30,20)][ContourLighting(%d,0.05)]",Lighting);
			else
				strcpy(szExec,"[Caption(\"DPLOTLIB Surface Plot\")][Contour3D(0)]");
			strcat(szExec,"[ContourGrid(0)][ContourAxes(1)][ContourMethod(0)][ContourLevels(20,-1,1)]"
				"[ContourScales(1,1,1)][ManualScale(-3,-3,3,3,-1,1)]"
                "[FontPoints(1,8)][FontPoints(2,12)][FontPoints(3,12)]"
                "[FontPoints(4,10)][FontPoints(5,10)][FontPoints(6,8)][FontPoints(8,10)]"
                "[ZAxisLabel(\"z\")][DocMaximize()]");
  
			if(hbmp)
			{
				DeleteObject(hbmp);
				hbmp = 0;
			}
			if(DocNum) dPlot_Command(DocNum, "[FileClose()]");
			DocNum = dPlot_Plot8(&DPlot,NULL,(double *)Node,szExec);
			if(DocNum)
			{
				SetBitmapToFrame();
				Is3D = 1;
			}
			t2 = GetTickCount();
			sprintf(szTime,"%d",t2-t1);
			SetDlgItemText(hwnd,DLG_TIMER,szTime);

			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),TRUE);

			return TRUE;

		case DLG_PLOT3DGRID:
			t1 = GetTickCount();
			/* Plot 3D points on a rectangular grid, generating a contour plot.*/
			GetDlgItemText(hwnd,DLG_NX,szText,sizeof(szText));
			NX = atoi(szText);
			GetDlgItemText(hwnd,DLG_NY,szText,sizeof(szText));
			NY = atoi(szText);

			if( NX < 1 ) {
				MessageBox(hwnd,"NX must be greater than 0.","Demo",MB_OK);
				return TRUE;
			}
			if( NY < 1 ) {
				MessageBox(hwnd,"NY must be greater than 0.","Demo",MB_OK);
				return TRUE;
			}

			/* Generate the data. - NP randomly placed points
			   in the range -PI < X < PI, -PI < Y < PI, Z=cos(X)*sin(Y) */
			if( NULL == (z = (double *)calloc( (NX+1)*(NY+1), sizeof(double)))) {
				MessageBox(hwnd,"Memory allocation failure.","Demo",
				   MB_OK | MB_ICONEXCLAMATION);
				return TRUE;
			}

			/*  Disable all command buttons to prevent multiple simultaneous calls to
				DPLOTLIB functions. Be sure we can't return without re-enabling those
				buttons. For more information see the "Timing Issues" topic in the
				DPLOTLIB Help file.
			*/
			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),FALSE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),FALSE);
			
			extents[0] = -PI;		// Xll
			extents[1] = -PI;		// Yll
			extents[2] =  PI;		// Xur
			extents[3] =  PI;		// Yur

			dX = 2*PI / NX;
			dY = 2*PI / NY;
			pz = z;
			for(i=0; i<=NX; i++)
			{
				XP = -PI + i*dX;
				for(j=0; j<=NY; j++)
				{
					YP = -PI + j*dY;
					*pz = (double)(cos(XP) * sin(YP));
					pz++;
				}
			}

			/* Initialize DPLOT structure to 0's: */
			memset(&DPlot,0,sizeof(DPlot));

			/* Fill in members of DPLOT structure */
			DPlot.Version	 = DPLOT_DDE_VERSION;
			DPlot.hwnd		 = (DWORD)hwnd;
			DPlot.DataFormat  = DATA_3D;
			DPlot.MaxCurves	 = NX;
			DPlot.MaxPoints	 = NY;
			DPlot.NumCurves	 = 1;
			DPlot.Scale		 = SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"DPLOT_GetBitmap Demo");
			strcpy(DPlot.XAxis,"X");
			strcpy(DPlot.YAxis,"Y");

			Is3DView = IsDlgButtonChecked(hwnd,DLG_3D);
			if(Is3DView)
				sprintf(szExec,"[Caption(\"DPLOTLIB Surface Plot\")]"
				               "[Contour3D(1)][ContourView(30,20)][ContourLighting(%d,0.05)]",Lighting);
			else
				strcpy(szExec,"[Caption(\"DPLOTLIB Surface Plot\")][Contour3D(0)]");
			strcat(szExec,"[ContourGrid(0)][ContourAxes(1)][ContourMethod(0)][ContourLevels(20,-1,1)]"
                "[ContourScales(1,1,1)][ManualScale(-3,-3,3,3,-1,1)]"
                "[FontPoints(1,8)][FontPoints(2,12)][FontPoints(3,12)]"
                "[FontPoints(4,10)][FontPoints(5,10)][FontPoints(6,8)][FontPoints(8,10)]"
                "[ZAxisLabel(\"z\")][DocMaximize()]");

			if(hbmp)
			{
				DeleteObject(hbmp);
				hbmp = 0;
			}
			if(DocNum) dPlot_Command(DocNum, "[FileClose()]");
			DocNum = dPlot_Plot8(&DPlot,extents,z,szExec);
			if(DocNum)
			{
				SetBitmapToFrame();
				Is3D = 1;
			}
			free(z);
			t2 = GetTickCount();
			sprintf(szTime,"%d",t2-t1);
			SetDlgItemText(hwnd,DLG_TIMER,szTime);

			EnableWindow(GetDlgItem(hwnd,DLG_PLOTXY),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DRAND),TRUE);
			EnableWindow(GetDlgItem(hwnd,DLG_PLOT3DGRID),TRUE);

			return TRUE;

		case DLG_3D:
			Is3DView = IsDlgButtonChecked(hwnd,DLG_3D);
			EnableWindow(GetDlgItem(hwnd,DLG_LIGHT_GROUP  ),Is3DView);
			EnableWindow(GetDlgItem(hwnd,DLG_LIGHT_NONE   ),Is3DView);
			EnableWindow(GetDlgItem(hwnd,DLG_LIGHT_GOURAUD),Is3DView);
			EnableWindow(GetDlgItem(hwnd,DLG_LIGHT_PHONG  ),Is3DView);
			return FALSE;

		case DLG_LIGHT_NONE:
		case DLG_LIGHT_GOURAUD:
		case DLG_LIGHT_PHONG:
			Lighting = LOWORD(wparam) - DLG_LIGHT_NONE;
			return FALSE;
		case DLG_EXITBUTTON:
			EndDialog( hwnd, TRUE );
			return TRUE;

		case IDCANCEL:
			EndDialog( hwnd, TRUE );
			return TRUE;

		default:
			return FALSE;

		}

	}
	
	return FALSE;
	
} /* PlotOptionsDlgProc */

// extern VOID	   Plot();

/*
* WinMain - initialization, message loop
*/
int PASCAL WinMain( HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline,
				   int cmdshow )
{
#ifdef __GNUC__
	HINSTANCE dll;
    if((dll = dPlot_Init()) == NULL) return 0;
#endif

	hbmp = (HBITMAP)NULL;
	DialogBox( this_inst,"DPLOTDEMO", NULL, PlotOptionsDlgProc );
	if(hbmp) DeleteObject(hbmp);

#ifdef __GNUC__
	FreeLibrary(dll);
#endif
	
	return 0;
	
} /* WinMain */

