(*
 converted from DPLOT.H
 by Serge Astaf'ev
 akagm@yandex.ru
*)

(*
 Version 4 of the DPLOT structure allows up to 100 curves, adds a 4th title line and increases
 the number of allowable characters in the title lines and X, Y axis labels from 80 to 200 characters.

 Version 2.1 or later of DPlot or DPlot Jr is required.
*)
unit
  DPlot_lib;

interface

uses
  Windows;

type
  TCHARS = packed array[0..1023] of char; //0..32767

const
  DPLOT_DLL_NAME    = 'DPLOTLIB.DLL';
  DPLOT_DDE_VERSION = 4;
  DPLOT_MAX_CURVES  = 100; // max # of curves

// Version 4 of the structure adds one title line and allows 200 characters in
// each title line and axis label.
type
  TDPLOT = packed record
    Version: DWORD;           // Caller must set this to DPLOT_DDE_VERSION
    hwnd: DWORD;              // handle of client application window
                              // (Use DWORD rather than HWND)
    DataFormat: DWORD;        // XY pairs, DX and Y, etc.
    MaxCurves: DWORD;         // maximum number of curves (must be <= 100)
                              // = NX for DataFormat = DATA_3D
                              //   ignore for DataFormat = DATA_3DR
    MaxPoints: DWORD;         // maximum number of points/curve
                              // = NY for DataFormat = DATA_3D
                              // = 3 * number of points for DATA_3DR
    NumCurves: DWORD;         // actual number of curves, always 1 for
                              //   DATA_3D or DATA_3DR
    Scale: DWORD;             // scaling code (Linear, Log, etc.)
    LegendX: single;          // left coord of legend, expressed as a ratio
                              //    of plot size (0->1)
    LegendY: single;          // top coord of legend
    NP: array[0..DPLOT_MAX_CURVES-1] of DWORD;          // actual number of points in each curve;
                              //   cannot exceed MaxPoints.
                              // For DATA_3DR files, return number of nodes in NP[0]
    LineType: array[0..DPLOT_MAX_CURVES-1] of DWORD;    // line types (see codes below)
    SymbolType: array[0..DPLOT_MAX_CURVES-1] of DWORD;  // symbol types (see codes below)
    SizeofExtraInfo: DWORD;   // Extra information following X,Y data

    Legend: packed array[0..DPLOT_MAX_CURVES] of packed array[0..79] of char; // Legend[0] is the caption for the legend.
                                          // Legend[n] is the legend for the n'th curve.
    LabelTxt: packed array[0..DPLOT_MAX_CURVES-1] of packed array[0..39] of char; // Strings displayed beside the last data point
                                              //   in a curve.
    Title: packed array[0..3] of packed array[0..199] of char;    // Three title lines.
    XAxis: packed array[0..199] of char;                   // X Axis label.
    YAxis: packed array[0..199] of char;                   // Y Axis label.

  end;


  TDPLOT_PLOTMETRICS = packed record
    size: DWORD;     // Size of this structure; should be set by the caller
    hll: integer;    // horizontal and
    vll: integer;    //  vertical coordinates of the lower left corner of
                     //  the plot, in pixels
    hur: integer;    // horizontal and
    vur: integer;    //  vertical coordinates of the upper right corner of
                     //  the plot, in pixels
    dummy: integer;  // for alignment
    xlo: double;     // value of x at the left plot extent
    ylo: double;     // value of y at the bottom plot extent
    xhi: double;     // value of x at the right plot extent
    yhi: double;     // value of z at the top plot extent
    stretch: array[0..2] of double; // scale factors for X, Y, and Z
    scale: double;
    azimuth: double; // view angles
    elevation: double;
    ylo2: double;    // Y extents of 2nd Y axis, if any
    yhi2: double;
  end;

const
  SIZE_TDPLOT             = sizeof(TDPLOT);
  SIZE_TDPLOT_PLOTMETRICS = sizeof(TDPLOT_PLOTMETRICS);

// Scaling codes:
const
  SCALE_LINEARX_LINEARY     =  1;
  SCALE_LINEARX_LOGY        =  2;
  SCALE_LOGX_LINEARY        =  3;
  SCALE_LOGX_LOGY           =  4;
  SCALE_TRIPARTITE          =  5;
  SCALE_LINEARX_PROBABILITY =  6;
  SCALE_GRAINSIZE_DIST      =  7;
  SCALE_POLAR               =  8;
  SCALE_BARCHART            =  9;
  SCALE_LOGX_PROBABILITY    = 10;
  SCALE_PROBX_LINEARY       = 11;
  SCALE_PROBX_LOGY          = 12;
  SCALE_PROBX_PROBY         = 13;
  SCALE_TRIANGLE_PLOT       = 14;
  SCALE_N185                = 15;
  SCALE_MERCATOR            = 16;

// Unit-specific scaling codes ... combine with above values using | operator
const
  UNITS_DEFAULT                = 0;
  UNITS_TRIPARTITE_INCHES      = UNITS_DEFAULT; // Velocity (Y) in inches/sec
  UNITS_TRIPARTITE_FEET        = $00000100;     //                 feet/sec
  UNITS_TRIPARTITE_MILLIMETERS = $00000200;     //                 mm/sec
  UNITS_TRIPARTITE_CENTIMETERS = $00000300;     //                 cm/sec
  UNITS_TRIPARTITE_METERS      = $00000400;     //                 meters/sec
  UNITS_GRAINSIZE_MILLIMETERS  = UNITS_DEFAULT; // Grain sizes (X) in mm
  UNITS_GRAINSIZE_INCHES       = $00000100;     //                    inches
  UNITS_POLAR_RADIANS          = UNITS_DEFAULT; // Rotation (X) in radians
  UNITS_POLAR_DEGREES          = $00000100;     //                 degrees
  UNITS_USERDEFINED            = $00007F00;

// Data organization
const
  DATA_XYXY        = 0;   // One or more sets of X,Y data
  DATA_DXY         = 1;   // One or more X,Y curves. Constant spacing in X and same number of points in all curves.
  DATA_XYYY        = 2;   // One or more X,Y curves. All curves have the same X values.
  DATA_3D          = 3;   // Z values on a rectangular grid
  DATA_3DR         = 4;   // Random X,Y,Z values
  DATA_IMAGE       = 5;   // Used only by DPlot - there's no way to SEND DPlot an image
  DATA_1D          = 6;   // One or more groups of Y values.
  DATA_3DS         = 7;   // 3D scatter plot
  DATA_4D          = 8;   // 4D surface plot
  DATA_4DS         = 9;   // 4D scatter plot

// Line styles:
const
  LINESTYLE_NONE          = 0;
  LINESTYLE_SOLID         = 1;
  LINESTYLE_LONGDASH      = 2;
  LINESTYLE_DOTTED        = 3;
  LINESTYLE_DASHDOT       = 4;
  LINESTYLE_MEDDASH       = 5;
  LINESTYLE_DASHDOTDOT    = 6;
  LINESTYLE_DASHDOTDOTDOT = 7;

// Symbol styles
const
  SYMBOLSTYLE_NONE        =  0;
  SYMBOLSTYLE_DOT         =  1;
  SYMBOLSTYLE_CROSS       =  2;
  SYMBOLSTYLE_ASTERISK    =  3;
  SYMBOLSTYLE_X           =  4;
  SYMBOLSTYLE_SQUARE      =  5;
  SYMBOLSTYLE_DIAMOND     =  6;
  SYMBOLSTYLE_TRIANGLE    =  7;
  SYMBOLSTYLE_OCTAGON     =  8;
  SYMBOLSTYLE_ITRIANGLE   =  9;
  SYMBOLSTYLE_HEXAGON     = 10;
  SYMBOLSTYLE_PENTAGON    = 11;
  SYMBOLSTYLE_STAR        = 12;
  SYMBOLSTYLE_FILL        = $00000100;   // May be combined with styles above

// TextNoteEx/TextNote3D flags
const
  N_FRAME      = $0002;    // Draw a rectangle around the note.
  N_OPAQUE     = $0004;    // Hide objects intersecting the note.
  N_BUTTON     = $0008;    // Note is button-activated
  N_DATASPACE  = $0010;    // Note location is in data space rather than the default plot space.
  N_HIDE       = $0020;    // Hides the note
  N_ARROW      = $0040;    // Draw a line w/ arrowhead from note to a reference point

// Function prototypes for DPLOTLIB.DLL

{
Plot/Plot8 functions
  x, y       -- static single/double arrays
  x[0], y[0] -- dynamic single/double arrays
}

function DPlot_3DBorder(DocNum, NumPts: integer; var doubleArray): integer; stdcall;
function DPlot_AddData(DocNum, DataType, NumPts, Curve: integer; var array1; var array2): integer; stdcall;
function DPlot_AddData8(DocNum, DataType, NumPts, Curve: integer; var array1; var array2): integer; stdcall;
function DPlot_AlwaysActivate(state: integer): integer; stdcall;
function DPlot_Command(DocNum: integer; commands: TCHARS): integer; stdcall;
function DPlot_Disconnect: integer; stdcall;
function DPlot_GetBitmap(DocNum, cx, cy: integer): integer; stdcall;
function DPlot_GetBitmapEx(DocNum, cx, cy: integer; var DPM: TDPLOT_PLOTMETRICS): integer; stdcall;
function DPlot_GetEnhMetaFile(DocNum: integer; cx, cy: single): integer; stdcall;
function DPlot_GetPlotMetrics(DocNum: integer; var DPM: TDPLOT_PLOTMETRICS): integer; stdcall;
function DPlot_GetVersion(var version: TCHARS): integer; stdcall;
function DPlot_MinVersion(digit1, digit2, digit3, digit4: integer): integer; stdcall;
function DPlot_Plot(var DPlot: TDPLOT; var x; var y; commands: TCHARS): integer; stdcall;
function DPlot_Plot8(var DPlot: TDPLOT; var x; var y; commands: TCHARS): integer; stdcall;
function DPlot_Plot8p(var DPlot: TDPLOT; var px: DWORD; var py: DWORD; commands: TCHARS): integer; stdcall;
function DPlot_PlotBitmap(var DPlot: TDPLOT; var x; var y; commands: TCHARS; cx, cy: integer): integer; stdcall;
function DPlot_PlotBitmap8(var DPlot: TDPLOT; var x; var y; commands: TCHARS; cx, cy: integer): integer; stdcall;
function DPlot_PlotToRect(var DPlot: TDPLOT; var x; var y; commands: TCHARS; h: HWND; var rc: TRECT): integer; stdcall;
function DPlot_PlotToRect8(var DPlot: TDPLOT; var x; var y; commands: TCHARS; h: HWND; var rc: TRECT): integer; stdcall;
function DPlot_Request(DocNum: integer; item: TCHARS; var value: TCHARS; var maxlength: DWORD): integer; stdcall;
function DPlot_SetErrorMethod(Method: integer): integer; stdcall;
function DPlot_SetTimeout(Timeout: integer): integer; stdcall;
function DPlot_Start(Minimize: integer; var WasActive: integer): integer; stdcall;
function DPlot_Stop: integer; stdcall;
function DPlot_YFromX(DocNum, CurveNum: integer; X: double; var Y: double): integer; stdcall;
function DPlot_ZFromXY(DocNum: integer; X, Y: double; var Z: double): integer; stdcall;

implementation

// Function prototypes for DPLOTLIB.DLL

function DPlot_3DBorder(DocNum, NumPts: integer; var doubleArray): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_AddData(DocNum, DataType, NumPts, Curve: integer; var array1; var array2): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_AddData8(DocNum, DataType, NumPts, Curve: integer; var array1; var array2): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_AlwaysActivate(state: integer): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_Command(DocNum: integer; commands: TCHARS): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_Disconnect: integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_GetBitmap(DocNum, cx, cy: integer): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_GetBitmapEx(DocNum, cx, cy: integer; var DPM: TDPLOT_PLOTMETRICS): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_GetEnhMetaFile(DocNum: integer; cx, cy: single): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_GetPlotMetrics(DocNum: integer; var DPM: TDPLOT_PLOTMETRICS): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_GetVersion(var version: TCHARS): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_MinVersion(digit1, digit2, digit3, digit4: integer): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_Plot(var DPlot: TDPLOT; var x; var y; commands: TCHARS): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_Plot8(var DPlot: TDPLOT; var x; var y; commands: TCHARS): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_Plot8p(var DPlot: TDPLOT; var px: DWORD; var py: DWORD; commands: TCHARS): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_PlotBitmap(var DPlot: TDPLOT; var x; var y; commands: TCHARS; cx, cy: integer): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_PlotBitmap8(var DPlot: TDPLOT; var x; var y; commands: TCHARS; cx, cy: integer): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_PlotToRect(var DPlot: TDPLOT; var x; var y; commands: TCHARS; h: HWND; var rc: TRECT): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_PlotToRect8(var DPlot: TDPLOT; var x; var y; commands: TCHARS; h: HWND; var rc: TRECT): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_Request(DocNum: integer; item: TCHARS; var value: TCHARS; var maxlength: DWORD): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_SetErrorMethod(Method: integer): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_SetTimeout(Timeout: integer): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_Start(Minimize: integer; var WasActive: integer): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_Stop: integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_YFromX(DocNum, CurveNum: integer; X: double; var Y: double): integer; stdcall; external DPLOT_DLL_NAME;
function DPlot_ZFromXY(DocNum: integer; X, Y: double; var Z: double): integer; stdcall; external DPLOT_DLL_NAME;

end.
