(*
  DPlot test example
  by Serge Astaf'ev
  akagm@yandex.ru
*)

{$APPTYPE CONSOLE}

uses
  Windows,
  SysUtils,
  Math,
  DPlot_lib;

const
  NC = 40;
  NP = 1001;
  NX = 61;
  NY = 41;
  NZ = 1000;


const
  NULL_PTR: pointer = nil;

type
  TXYZ = packed record
    x, y, z: double;
  end;

  TXYZA = packed record
    x, y, z, a: double;
  end;


var
  DPlot: TDPLOT;

  x: array[1..NP] of double;
  y: array[1..2*NP] of double;
  z: array[1..NX*NY] of double;
  pX, pY, pZ: array of double;

  xx, yy, zz, dummy, x0, y0, xlo, xhi, ylo, yhi, side, sum, tmp: double;
  radius, alpha, theta, phi, theta_inc: double;
  xmin, xmax, ymin, ymax, zmin, zmax: double;
  Vc, Vs, Vnet, Vf: double;

  extents: array[1..4] of double;

  numCurves, numPoints, numX, numY, numZ: integer;
  lon, lat, nStep: integer;

  Node: array[1..NZ] of TXYZ;
  pNode: array of TXYZ;
  pNodeA: array of TXYZA;

  Star: array[0..9] of TXYZ;
  border: array[0..9] of array[0..1] of double;

  option, i, j, k: integer;
  docNum: integer;

  cmdStr, charArray: TCHARS;
  str: string;

  dwSize: DWORD;
  szTopics: TCHARS;

  retval: integer;

  doc1Acive: boolean = false;

function DEGTORAD(x: extended): extended;
begin
  DEGTORAD := (x/57.295779513082320876798154814105);
end;

// main
begin
  randomize;
  writeln;
  docNum := DPlot_GetVersion(chararray);
  if (docNum < 1) then begin
    writeln('can''t find any DPlot package in your system'#13#10);
    halt(1);
  end;

  if (docNum = 1) then
    str := 'DPlot'
  else
    str := 'DPlot Jr';
  writeln(str + ' v.' + chararray + #13#10);

  repeat

    fillchar(DPlot, SIZE_TDPLOT, #0);
    DPlot.Version := DPLOT_DDE_VERSION;

    writeln('Enter 0 to exit');
    writeln('      1 for XY plot example');
    writeln('      2 for contour plot of gridded data');
    writeln('      3 for contour plot of randomly-spaced points');
    writeln('      4 for box-and-whisker example');
    writeln('      5 Another XY plot (30 curves)');
    writeln('      6 3D scatter plot');
//    writeln('      7 Yet another XY plot (world pop. estimate)');
//    writeln('      8 Get DPlot/DPlot Jr version number');
    writeln('      7 Get peak values from the document (at least one window from any other demo must be active)');
    writeln('      8 Surface plot with concave border');
    writeln('      9 4D surface plot');
    writeln('     10 4D scatter plot');
    writeln('     11 Another contour plot');
    writeln('     12 2D vector plot');
//    writeln('     xx Make movie frames of a 3D/4D plot in 3D view');
    writeln('     13 Collatz Conjecture (transparent symbols)');
    writeln('     14 forum''s example: Bar Chart');
    writeln('     15 forum''s example: Contour');
    writeln('     16 forum''s example: Unit hemisphere');
    writeln('     17 forum''s example: List all available DPlot DDE topics');

    writeln('? ');

    readln(option);
    writeln;

    case (option) of
      0 : break;
      1 :
{
  Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
  In this case we've used DATA_XYYY (one X array for one or more Y arrays).

  Since the X values are evenly spaced we could also use
  DATA_DXY -- X has only 2 elements, DX and X0

  And you can ALWAYS use
  DATA_XYXY -- Each curve uses its own X array. This is the only
               option available if the curves have different X values or
               a different number of points.
}
        begin
          for i := 1 to NP do begin
            x[i]    := (4.0*i)/(NP-1);
            y[i]    := sin(PI*x[i]);
            y[i+NP] := cos(PI*x[i]);
          end;

          setlength(pX, NP);
          setlength(pY, 2*NP);

          for i := 0 to NP-1 do begin
            px[i]    := x[i+1];
            py[i]    := y[i+1];
            py[i+NP] := y[i+NP+1];
          end;

          DPlot.hwnd        := 0;
          DPlot.DataFormat  := DATA_XYYY;
          DPlot.MaxCurves   := 2;
          DPlot.MaxPoints   := NP;
          DPlot.NumCurves   := 2;
          DPlot.Scale       := SCALE_LINEARX_LINEARY;
          DPlot.LegendX     := 0.05;
          DPlot.LegendY     := 0.05;
          DPlot.NP[0]       := NP;
          DPlot.NP[1]       := NP;
          DPlot.LineType[0] := LINESTYLE_SOLID;
          DPlot.LineType[1] := LINESTYLE_DOTTED;//LINESTYLE_LONGDASH;

          DPlot.Legend[1]   := 'sin({\sp}x)';
//                                   {\s is DPlot-speak for "use symbol font"
          DPlot.Legend[2]   := 'cos({\sp}x)';
          DPlot.Title[0]    := 'Data sent to DPlot via DPLOTLIB.DLL (static arrays)';

          DPlot.XAxis       := 'x';
          DPlot.YAxis       := 'y';

{
  The command string can be as complex as you like, limited to 32K characters.
  As an alternative, if your program will be producing many similar plots then
  you might prefer to create a preferences file, edit the file with a text
  editor to remove unwanted entries, and use the [GetPreferences("filename")] command.
}
//!as!        docNum = DPlot_Plot(loc(DPlot),x(1),y(1),
//          docNum := DPlot_Plot8(DPlot, x[1], y[1],
          docNum := DPlot_Plot8(DPlot, x, y,
//                             '[WindowCascade()]' +
                             '[ManualScale(0,-1.25,4,1.25)]' +
                             '[TickInterval(1,0.5,0.25)]' +
                             '[Caption("DPLOTLIB XY (static arrays)")]' +
//                             '[DocMaximize()]' +
                             '[ClearEditFlag()]'#0);

          doc1Acive := true;

          DPlot.Title[0]    := 'Data sent to DPlot via DPLOTLIB.DLL (dynamic arrays)';

          docNum := DPlot_Plot8(DPlot, px[0], py[0],
                             '[WindowCascade()]' +
                             '[ManualScale(0,-1.25,4,1.25)]' +
                             '[TickInterval(1,0.5,0.25)]' +
                             '[Caption("DPLOTLIB XY (dynamic arrays)")]' +
//                             '[DocMaximize()]' +
                             '[ClearEditFlag()]'#0);

//          DPlot_Command(1, '[WindowCascade()]'#0);


          setlength(px, 0);
          setlength(py, 0);


//        break;
        end;

      2 :
// Contour plot of Z values on a rectangular grid.  z = sin(x)*cos(y)
        begin

//       extents are, in order, xlo, ylo, xhi, yhi
          extents[1] := -3.;
          extents[2] := -2.;
          extents[3] :=  3.;
          extents[4] :=  2.;
          for i := 1 to Nx do begin
            xx := extents[1] + ((extents[3]-extents[1])*i)/(Nx-1);
            for j := 1 to Ny do begin
              yy := extents[2] + ((extents[4]-extents[2])*j)/(Ny-1);
              z[(i-1)*Ny+j] := sin(xx)*cos(yy);
            end;
          end;

          DPlot.hwnd       := 0;
          DPlot.DataFormat := DATA_3D;

//       MaxCurves and MaxPoints members are used for the size of the grid, in grid cells
         DPlot.MaxCurves  := Nx-1;
         DPlot.MaxPoints  := Ny-1;
         DPlot.NumCurves  := 1;
         DPlot.Scale      := SCALE_LINEARX_LINEARY;
         DPlot.Title[0]   := 'Data sent to DPlot via DPLOTLIB.DLL';
         DPlot.Title[1]   := 'z = sin(x)*cos(y)';
         DPlot.XAxis      := 'x';
         DPlot.YAxis      := 'y';

         docNum := DPlot_Plot8(DPlot, extents, z,
                '[Caption("DPLOTLIB Contour Test 1")]' +
                '[Contour3D(1)][ContourMethod(0)]' +
                '[ContourGrid(0)][ContourAxes(1)]' +
                '[ContourView(30,20)][ContourLevels(20,-1,1)]' +
                '[ContourScales(1,1,1)][ContourLighting(2,0.05)]' +
                '[FontPoints(1,10)][FontPoints(6,10)]' +
                '[ZAxisLabel("z")][DocMaximize()]'#0);

        end;


      3 :
// Contour plot with randomly spaced points
        begin

          for i := 1 to Nz do begin
            Node[i].x := -PI + 2*PI*random;
            Node[i].y := -PI + 2*PI*random;
            Node[i].z := sin(Node[i].x) * cos(Node[i].y);
          end;

          DPlot.hwnd       := 0;
          DPlot.DataFormat := DATA_3DR;
          DPlot.MaxCurves  := 1;
          DPlot.MaxPoints  := Nz;
          DPlot.NumCurves  := 1;
          DPlot.Scale      := SCALE_LINEARX_LINEARY;
          DPlot.Title[0]   := 'Data sent to DPlot via DPLOTLIB.DLL';
          DPlot.Title[1]   := 'z = sin(x)*cos(y)';
          DPlot.XAxis      := 'x';
          DPlot.YAxis      := 'y';


          docNum := DPlot_Plot8(DPlot, NULL_PTR, Node[1].x,
               '[Caption("DPLOTLIB Contour Test 2")]' +
               '[Contour3D(0)][ContourGrid(0)]' +
//             '[ContourMethod(1)]' +
               '[ContourLevels(21,-1,1)]' +
               '[FontPoints(1,10)][FontPoints(6,10)]' +
               '[DocMaximize()]'#0);

        end;

      4 :
// Box-and-whisker plot - 3 groups
        begin

          DPlot.hwnd       := 0;
          DPlot.DataFormat := DATA_1D;
          DPlot.MaxCurves  := 3;
          DPlot.MaxPoints  := 50;
          DPlot.NumCurves  := 3;
          DPlot.LegendX    := 0.02;
          DPlot.LegendY    := 0.7;
          DPlot.Scale      := SCALE_LINEARX_LINEARY;
          DPlot.Title[0]   := 'Data sent to DPlot via DPLOTLIB.DLL';
          DPlot.XAxis      := ' ';
          DPlot.YAxis      := 'Amplitude';

          for i := 1 to DPlot.NumCurves do begin
            DPlot.NP[i-1] := DPlot.MaxPoints;
            DPlot.Legend[i] := 'Group '#0;// + char(64+i) + #0;
            DPlot.Legend[i, 6] := char(64+i);
            DPlot.Legend[i, 7] := #0;
          end;

//        Generate group data, centered around 5.0 with progressively larger variations
          k := 1;
          for i := 1 to DPlot.NumCurves do begin
            for j := 1 to DPlot.NP[i-1] do begin
              y[k] := 5.0 + (i+1)*(-0.5+random);
              inc(k);
            end;
          end;

          docNum := DPlot_Plot8(DPlot, NULL_PTR, y[1],
                '[Caption("DPLOTLIB Box-and-whisker Test")]' +
                '[Stat_PlotType(0)][Stat_GroupInfo(15,4)]' +
                '[Stat_BWOptions(1,1,0,1,1.5,2,3)]' +
                '[Stat_GrandMean(3,30)]' +
                '[FontPoints(1,10)][FontPoints(6,10)]' +
                '[DocMaximize()]'#0);

        end;


      5 :
// Simulate a square wave with a sum of sine waves.
// First 29 curves are sine waves, last curve is the addition of 1st 29.

        begin

          setlength(px, NP);
          setlength(py, NP*NC);

          for i := 0 to NP-1 do begin
            px[i] := (2*PI*i)/(NP-1);
            sum := 0.0;
            for j := 0 to NC-1 do begin
              py[i+j*NP] := sin((j*2+1)*px[i])/(j*2+1);
              sum := sum + py[i+j*NP];
            end;
            py[i+(NC-1)*NP] := (4.0/PI) * sum;
          end;


         DPlot.DataFormat := DATA_XYYY;
         DPlot.MaxCurves  := NC; // Must be >= number of curves we plot
         DPlot.MaxPoints  := NP; // Anything >= NP will do
         DPlot.NumCurves  := NC;
         DPlot.Scale      := SCALE_LINEARX_LINEARY;
         DPlot.LegendX    := 0.05;
         DPlot.LegendY    := 0.7;

         for j := 0 to NC-1 do begin
           DPlot.NP[j] := NP;
           DPlot.LineType[j] := LINESTYLE_SOLID;
           if (j < NC-1) then begin
//             sprintf(DPlot.Legend[j+1],"sin(%dx)/%d",j*2+1,j*2+1);
             str := format('sin(%dx)/%d', [j*2+1, j*2+1]) + #0;

             move(str[1], DPlot.Legend[j+1], length(str));
           end;
         end;

         DPlot.Legend[NC] := '4/{\sp}(sin(x)+sin(3x)/3+sin(5x)/5+...)';
         DPlot.Title[0]   := 'Square wave approximation';
         DPlot.Title[1]   := 'z = sin(x)*cos(y)';
         DPlot.XAxis      := 'x';
         DPlot.YAxis      := 'y';


         docNum := DPlot_Plot8(DPlot, px[0], py[0],
           '[ManualScale(0,-1.25,6.5,1.25)][TickInterval(1,0.5,0.25)]' +
           '[LineWidth(-2,0)][Caption("DPLOTLIB XY Test")][DocMaximize()]' +
           '[ClearEditFlag()]'#0);

          setlength(px, 0);
          setlength(py, 0);

        end;

      6 :
// 3D scatter plot - random points on 3 spheres.
        begin
          // Number of points in each of the NC data sets. Data sets CAN have
          // differing numbers of points. In either case, the Node array is
          // packed with no unused nodes. For example the first point in the
          // 2nd data set should immediately follow the last point in the 1st data set.
          numCurves := 3;

          setlength(pNode, numCurves*NZ);

          xmin := 0.0;
          xmax := 0.0;
          ymin := 0.0;
          ymax := 0.0;
          zmin := 0.0;
          zmax := 0.0;
          theta_inc := 2.0 * sqrt(NZ);

          for j := 0 to numCurves-1 do begin

            xx := -5 + 10. * random;
            yy := -5 + 10. * random;
            zz := -5 + 10. * random;
            radius := 4. + 4. * random;
            xmin := min(xmin, xx-radius);
            xmax := max(xmax, xx+radius);
            ymin := min(ymin, yy-radius);
            ymax := max(ymax, yy+radius);
            zmin := min(zmin, zz-radius);
            zmax := max(zmax, zz+radius);
            for i := j*NZ to (j+1)*NZ-1 do begin
              phi := -PI/2 + PI*(i-j*NZ)/NZ;
              theta := theta_inc * PI*(i-j*NZ)/NZ;
              pNode[i].x := xx + radius*cos(theta)*cos(phi);
              pNode[i].y := yy + radius*sin(theta)*cos(phi);
              pNode[i].z := zz + radius*sin(phi);
            end;
//            sprintf(DPlot.Legend[j+1],"Sphere %d",j+1);
            str := format('Sphere %d', [j+1]) + #0;
            move(str[1], DPlot.Legend[j+1], length(str));
            DPlot.NP[j] := NZ;
          end;//j

          DPlot.DataFormat := DATA_3DS;
          DPlot.MaxCurves  := numCurves;
          DPlot.MaxPoints  := NZ;
          DPlot.NumCurves  := numCurves;
          DPlot.Scale      := SCALE_LINEARX_LINEARY;
          DPlot.Legend[0]  := 'Legend';
          DPlot.Title[0]   := 'DPLOTLIB - Points on Spheres';
          DPlot.XAxis      := 'x';
          DPlot.YAxis      := 'y';

          xmin := floor(xmin);
          xmax := ceil(xmax);
          ymin := floor(ymin);
          ymax := ceil(ymax);
          zmin := floor(zmin);
          zmax := ceil(zmax);

          str := '[Caption("DPLOTLIB 3D scatter plot")][TickInterval(1,1,1,1)][ManualScale(%g,%g,%g,%g,%g,%g)]' +
                 '[ColorBasis(1)][ContourColorScheme(2)][ContourCustomColors(255,65535,16711680)][ContourScales(1,1,1)]' +
                 '[FontPoints(1,10)][FontPoints(6,10)][DocMaximize()]';
          str := format(str, [xmin, ymin, xmax, ymax, zmin, zmax]) + #0;

          move(str[1], cmdStr[0], length(str));

          docNum := DPlot_Plot8(DPlot, NULL_PTR, pNode[0], cmdStr);

          setlength(pNode, 0);

        end;

      7 :
        begin
          dwSize := sizeof(cmdStr);
          retval := DPlot_Request(1, 'ListPeaks', cmdStr, dwSize);
          if (retval > 0) then
            writeln(cmdStr)
          else if (retval = -3) then
            writeln('insufficient buffer size, ', dwSize, ' required.')
          else
            writeln('return value of DPlot_Request = ', retval);
        end;

      8 :
// surface plot with concave edges
        begin

          x0 := 0;
          y0 := 0;
          side := 10.0;

          Star[0].x := x0 - 0.5*side;
          Star[0].y := y0 - 0.5*side*tan(DEGTORAD(54));
          Star[0].z := 0.0;
          Star[1].x := x0;
          Star[1].y := Star[0].y + 0.5*side*tan(DEGTORAD(36));
          Star[1].z := 1.0;
          Star[2].x := x0 + 0.5*side;
          Star[2].y := Star[0].y;
          Star[2].z := 0.0;
          Star[3].x := Star[2].x - 0.5*side*(sin(DEGTORAD(18))/cos(DEGTORAD(36)));
          Star[3].y := Star[2].y + 0.5*side*(cos(DEGTORAD(18))/cos(DEGTORAD(36)));
          Star[3].z := 1.0;
          Star[4].x := Star[2].x + side*sin(DEGTORAD(18));
          Star[4].y := Star[2].y + side*cos(DEGTORAD(18));
          Star[4].z := 0.0;
          Star[5].x := x0 + 0.5*side*(sin(DEGTORAD(18))/cos(DEGTORAD(36)));
          Star[5].y := Star[4].y;
          Star[5].z := 1.0;
          Star[6].x := x0;
          Star[6].y := Star[4].y + side*sin(DEGTORAD(36));
          Star[6].z := 0.0;
          Star[7].x := 2*x0 - Star[5].x;
          Star[7].y := Star[5].y;
          Star[7].z := 1.0;
          Star[8].x := 2*x0 - Star[4].x;
          Star[8].y := Star[4].y;
          Star[8].z := 0.0;
          Star[9].x := 2*x0 - Star[3].x;
          Star[9].y := Star[3].y;
          Star[9].z := 1.0;

          DPlot.DataFormat := DATA_3DR;
          DPlot.MaxCurves  := 1;
          DPlot.MaxPoints  := 10;
          DPlot.NumCurves  := 1;
          DPlot.Scale      := SCALE_LINEARX_LINEARY;

          DPlot.Title[0] := 'Surface Plot with Concave Edges';
          DPlot.Title[1] := 'See {\iDefine Boundary} command on {\iOptions} menu';
          DPlot.XAxis := 'x';
          DPlot.YAxis := 'y';

          docNum := DPlot_Plot8(DPlot, NULL_PTR, Star[0].x, '[Caption("DPLOTLIB Concave Surface Plot")]' +
                      '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)][ContourLevels(20,0,1)][DocMaximize()]'#0);
          if (docNum > 0) then begin
            for i := 0 to 9 do begin
              border[i, 0] := Star[i].x;
              border[i, 1] := Star[i].y;
            end;
            DPlot_3DBorder(docNum, 10, border[0, 0]);
          end;


        end;

      9 :
// 4D Contour plot of Z values on a rectangular grid.
        begin

          numX := 121; // Rows and columns of data
          numY := 121;
          setlength(pNodeA, numX*numY);

          k := 0;
          for i := 0 to numX-1 do begin
            xx := 5 + 30*i/(numX-1);
            for j := 0 to numY-1 do begin
              pNodeA[k].x := xx;
              yy := 5 + 30*j/(numY-1);
              pNodeA[k].y := yy;
              if ((xx = 20.0) and (yy = 20.0)) then
                zz := sin(1.0)
              else
                zz := sin(sqrt(sqr(xx-20.0) + sqr(yy-20.0)))/sqrt(sqr(xx-20.0) + sqr(yy-20.0));
              zz := zz + 1.5;
              zz := 100. * zz;
              pNodeA[k].z := zz;
              pNodeA[k].a := 10.0 * (1.1 + sin((xx-20.0)/5.0)*cos((yy-20.0)/10.0));
              inc(k);
            end;
          end;

          DPlot.DataFormat := DATA_4D;
          DPlot.MaxCurves  := 1;
          DPlot.MaxPoints  := numX*numY;
          DPlot.NumCurves  := 1;
          DPlot.Scale      := SCALE_LINEARX_LINEARY;

          DPlot.Title[0] := 'Data sent to DPlot via DPLOTLIB.DLL';
          DPlot.Title[1] := 'z = 100 * (sin(sqrt((x-20.){\u2}+(y-20.){\u2}))/sqrt((x-20.){\u2}+(y-20.){\u2})+1.5)';
          DPlot.Title[2] := 'a = 10. * (1.1 + sin((x-20.)/5.)*cos((y-20.)/10.))';
          DPlot.XAxis := 'x';
          DPlot.YAxis := 'y';

          docNum := DPlot_Plot8(DPlot, NULL_PTR, pNodeA[0].x, '[Caption("DPLOTLIB 4D Surface Plot Test")]' +
                      '[Contour3D(1)][ContourMethod(0)][ContourGrid(0)][ContourAxes(1)][ContourView(118,28)]' +
                      '[ContourLevels(22,0,22)][ContourScales(1,1,0.2)][ContourLighting(0,0.05)]' +
                      '[ContourColorScheme(2)][ContourCustomColors(0,0x00FF047D,0x000B1EB3,0x000068DC,0x0000FAFE)]' +
                      '[FontPoints(1,10)][FontPoints(4,14)][FontPoints(5,14)][FontPoints(6,10)][FontPoints(8,14)]' +
                      '[ZAxisLabel("z")][AAxisLabel("Amplitude")][DocMaximize()]'#0);

          setlength(pNodeA, 0);

        end;


      10 :
// 4D scatter plot - random points on 3 spheres.
        begin
          numZ := 1000; // Number of points in each of the NC data sets. Data sets CAN have
                        // differing numbers of points. In either case, the Node array is
                        // packed with no unused nodes. For example the first point in the 2nd
                        // data set should immediately follow the last point in the 1st data set.
          numCurves := 3;
          setlength(pNodeA, numCurves*NZ);

          xmin := 0.0;
          xmax := 0.0;
          ymin := 0.0;
          ymax := 0.0;
          zmin := 0.0;
          zmax := 0.0;;
          theta_inc := 2.0*sqrt(numZ);

          for j := 0 to numCurves-1 do begin
            xx := -10. + 20.*random;
            yy := -10. + 20.*random;
            zz := -10. + 20.*random;
            radius := 4. + 4.*random;
            xmin := min(xmin, xx-radius);
            xmax := max(xmax, xx+radius);
            ymin := min(ymin, yy-radius);
            ymax := max(ymax, yy+radius);
            zmin := min(zmin, zz-radius);
            zmax := max(zmax, zz+radius);

            for i := j*numZ to (j+1)*numZ-1 do begin
              phi := -PI/2 + PI*(i-j*Nz)/numZ;
              theta := theta_inc*PI*(i-j*Nz)/numZ;
              pNodeA[i].x := xx + radius*cos(theta)*cos(phi);
              pNodeA[i].y := yy + radius*sin(theta)*cos(phi);
              pNodeA[i].z := zz + radius*sin(phi);
              pNodeA[i].a := pNodeA[i].x;
            end;

//            sprintf(DPlot.Legend[j+1],"Sphere %d",j+1);
            str := format('Sphere %d', [j+1]) + #0;
            move(str[1], DPlot.Legend[j+1], length(str));
            DPlot.NP[j] := numZ;
          end;

          DPlot.DataFormat := DATA_4DS;
          DPlot.MaxCurves  := numCurves;
          DPlot.MaxPoints  := numZ;
          DPlot.NumCurves  := numCurves;
          DPlot.Scale      := SCALE_LINEARX_LINEARY;
          DPlot.Legend[0]  := 'Legend';
          DPlot.Title[0]   := 'DPLOTLIB - Points on Spheres';
          DPlot.Title[1]   := 'Amplitude=X';
          DPlot.XAxis      := 'x';
          DPlot.YAxis      := 'y';

          xmin := floor(xmin);
          xmax := ceil(xmax);
          ymin := floor(ymin);
          ymax := ceil(ymax);
          zmin := floor(zmin);
          zmax := ceil(zmax);

          str := '[Caption("DPLOTLIB Test 12")][TickInterval(1,1,1,1)][ManualScale(%g,%g,%g,%g,%g,%g)][ColorBasis(0)]' +
                 '[ContourColorScheme(2)][ContourCustomColors(255,65535,16711680)][ContourScales(1,1,1)][FontPoints(1,10)]' +
                 '[FontPoints(6,10)][DocMaximize()][SymbolType(-1,288)][SymbolSize(1,100)][SymbolSize(2,150)][SymbolSize(3,200)]';
          str := format(str, [xmin, ymin, xmax, ymax, zmin, zmax]) + #0;
          move(str[1], cmdStr[0], length(str));

          docNum := DPlot_Plot8(DPlot, NULL_PTR, pNodeA[0].x, cmdStr);
          setlength(pNodeA, 0);
        end;

      11 :
        begin

          numX := 361; // Columns
          numY := 161; // Rows
          setlength(pZ, numX*numY);

          extents[1] := 0.0;   // xlo
          extents[2] := 0.0;   // ylo
          extents[3] := 360.0; // xhi
          extents[4] := 1.0;   // yhi

          for i := 0 to numX-1 do begin

            xx := extents[1] + ((extents[3]-extents[1])*i)/(numX-1);
            for j := 0 to numY-1 do begin

              yy := extents[2] + ((extents[4]-extents[2])*j)/(numY-1);
              Vc := 1.0 - yy*cos(DEGTORAD(xx));
              Vs := yy*sin(DEGTORAD(xx));
              Vnet := sqrt(Vc*Vc + Vs*Vs);
// Borland C doesn't handle atan2(0,0). Others do.
              if (Vnet > 0) then
                theta := ArcTan2(Vs*Vs, Vc*Vc)
              else
                theta := 0.;
              Vf := Vnet*Vnet * abs(cos(theta));
              if (Vc < 0) then
                pZ[i*numY+j] := -Vf
              else
                pZ[i*numY+j] := Vf;
            end;
          end;

          DPlot.DataFormat := DATA_3D;
          DPlot.MaxCurves  := numX-1;   // Size of the grid, in grid cells
          DPlot.MaxPoints  := numY-1;
          DPlot.NumCurves  := 1;
          DPlot.Scale      := SCALE_LINEARX_LINEARY;

          DPlot.Title[0] := 'Relative Drag Force for a Spherical Object';
          DPlot.Title[1] := 'Drag{\dactual}/Drag{\dno wind}\n';
          DPlot.XAxis    := 'Wind Direction, degrees';
          DPlot.YAxis    := 'Wind Speed/Ground Speed';

          docNum := DPlot_plot8(DPlot, extents, pZ[0], '[Caption("DPLOTLIB Contour Test 13")]' +
             '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)][ContourAxes(1)]' +
             '[ContourLevels(40,0,4)][ContourScales(1,360,1)][ContourLighting(2,0.05)]' +
             '[ManualScale(0,0,360,1,0,4)][TickInterval(1,10,0.1,1)]' +
             '[FontPoints(1,10)][FontPoints(6,10)][ZAxisLabel("z")]' +
             '[XYLabel(0,1,0,"Tailwind")][XYLabel(180,1,0,"Headwind")][XYLabel(360,1,0,"Tailwind")]' +
             '[DocMaximize()][ClearEditFlag()]'#0);

           setlength(pZ, 0);
        end;


      12 :
        begin

          numX := 37;  // Columns
          numY := 16;  // Rows
          numPoints := numX*numY;
          setlength(pX, numPoints);
          setlength(pY, 3*numPoints);

          xlo := 0.0;    // xlo
          ylo := 0.0;    // ylo
          xhi := 360.0;  // xhi
          yhi := 1.5;    // yhi
          k := 0;
          for i := 0 to numX-1 do begin
            xx := xlo + ((xhi-xlo)*i) / (numX-1);
            for j := 0 to numY-1 do begin
              yy := ylo + ((yhi-ylo)*j) / (numY-1);
              pX[k] := xx;
              pY[k] := yy;
              alpha := xx;
              if (alpha < 0) then alpha := alpha + 360.0;
              pY[k+numPoints] := PI - ArcTan2(yy*sin(DEGTORAD(alpha)), 1.0-yy*cos(DEGTORAD(alpha)));
              if (pY[k+numPoints] < 0.0) then pY[k+numPoints] := pY[k+numPoints] + 2.0*PI;
              if (pY[k+numPoints] >= 2.0*PI) then pY[k+numPoints] := pY[k+numPoints] - 2.0*PI;
              pY[k+2*numPoints] := sqrt(1.0-2*yy*cos(DEGTORAD(alpha))+yy*yy);
              inc(k);
            end;
          end;

          DPlot.DataFormat  := DATA_XYYY;
          DPlot.MaxCurves   := 3;
          DPlot.MaxPoints   := numPoints;
          DPlot.NumCurves   := 3;
          DPlot.NP[0] := numPoints;
          DPlot.NP[1] := numPoints;
          DPlot.NP[2] := numPoints;
          DPlot.SymbolType[0] := SYMBOLSTYLE_DOT;
          DPlot.SymbolType[1] := SYMBOLSTYLE_DOT;
          DPlot.SymbolType[2] := SYMBOLSTYLE_DOT;
          DPlot.Scale := SCALE_LINEARX_LINEARY;

          DPlot.Title[0] := 'Air Speed/Ground Speed\n';
          DPlot.XAxis    := 'Wind Direction, degrees';
          DPlot.YAxis    := 'Wind Speed/Ground Speed';

          docNum := DPlot_plot8(DPlot, pX[0], pY[0], '[Caption("DPLOTLIB Contour Test 14")]' +
            '[ManualScale(-10,-0.1,370,1.6)]' +
            '[TickInterval(1,10,0.1)]' +
            '[FontPoints(1,10)][FontPoints(6,10)][AxisType(2)]' +
            '[XYLabel(0,1,0,"Tailwind")][XYLabel(90,1,0,"Left")][XYLabel(180,1,0,"Headwind")]' +
            '[XYLabel(270,1,0,"Right")][XYLabel(360,1,0,"Tailwind")]' +
            '[VectorPlot(1,2,3,0x0000FF00,0x000000FF,100,500)][VectorPlotOptions(6,150,20)]' +
            '[DocMaximize()][ClearEditFlag()]'#0);

          setlength(pX, 0);
          setlength(pY, 0);

        end;

      13 :
// Collatz Conjecture
        begin

          numPoints := 100000;
          setlength(pX, numPoints);
          setlength(pY, numPoints);
          for i := 1 to numPoints do begin
            j := 0;
            k := i;
            while (k <> 1) do begin
              if not odd(k) then
                k := k shr 1
              else
                k := k*3 + 1;
              inc(j);
            end;
            pX[i-1] := i;
            pY[i-1] := j;
          end;


          DPlot.DataFormat  := DATA_XYXY;
          DPlot.MaxCurves   := 1;         // Must be >= number of curves we plot
          DPlot.MaxPoints   := numPoints; // Anything >= NP will do
          DPlot.NumCurves   := 1;
          DPlot.Scale       := SCALE_LINEARX_LINEARY;
          DPlot.LegendX     := 0.05;
          DPlot.LegendY     := 0.05;
          DPlot.NP[0]       := numPoints;
          DPlot.LineType[0] := LINESTYLE_NONE;
          DPlot.SymbolType[0] := 288;

          DPlot.Title[0] := 'Collatz Conjecture';
          DPlot.YAxis    := 'Number of iterations';

          str := '[Caption("DPLOTLIB Test")][DocMaximize()]' +
                 '[ClearEditFlag()][SymbolSizeUnits(0)][SymbolSize(1,100)][SymbolOpacity(1,10)]' +
                 '[TextNoteEx("{\bCollatz Conjecture:}\n' +
                             'Take any natural number {\in}. If {\in} is even, divide it by 2\n' +
                             'to get {\in}/2, if {\in} is odd multiply it by 3 and add 1 to\n' +
                             'obtain 3{\in} + 1. Repeat the process indefinitely. The\n' +
                             'conjecture is that no matter what number you start with,\n' +
                             'you will always eventually reach 1. This example plots\n' +
                             'the number of iterations required to reach 1 for all\n' +
                             'numbers between 1 and %d (and also shows off \n' +
                             'DPlot''s partially transparent symbols feature).",0,0,0,0)]';

          str := format(str, [numPoints]) + #0;

          move(str[1], cmdStr[0], length(str));
          docNum := DPlot_Plot8(DPlot, pX[0], pY[0], cmdStr);

          setlength(pX, 0);
          setlength(pY, 0);

        end;

// Forum's examples

      14 :
        begin
          numPoints := 2;
          setlength(pX, numPoints);
          setlength(pY, 3*numPoints);

          pX[0] := 1;
          pX[1] := 0;
          pY[0] := 55;
          pY[1] := 23;
          pY[2] := 65;
          pY[3] := 40;
          pY[4] := 80;
          pY[5] := 10;

          DPlot.DataFormat := DATA_DXY;
          DPlot.MaxCurves  := 3;  // Must be >= number of curves we plot
          DPlot.MaxPoints  := numPoints; // Anything >= numPoints will do
          DPlot.NumCurves  := 3;
          DPlot.Scale      := SCALE_BARCHART;
          DPlot.NP[0]      := numPoints;
          DPlot.NP[1]      := numPoints;
          DPlot.NP[2]      := numPoints;
          DPlot.Legend[1]  := '2010';
          DPlot.Legend[2]  := '2011';
          DPlot.Legend[3]  := '2012';
          DPlot.Title[0]   := 'Bar Chart Question from Forums';

          docNum := DPlot_plot8(DPlot, pX[0], pY[0], '[Caption("Bar Chart Question from Forums")]' +
            '[ClearEditFlag()][BarChartStyle(1)][BarWidth(1,0.75)]' +
            '[XYLabel(0.5,0,0,"Apples")]' +
            '[XYLabel(1.5,0,0,"Pears")]' +
            '[NumberFormat(0,12)]' +
            '[ManualScale(0,0,2,90)]' + // This isn't strictly necessary, but w/o it the y=80 bar goes to the top of the plot.
            '[Size(1,6,6,1)]' +
// Standard Excel colors, just because:
            '[BkPlotColor(0xC0C0C0)][AxisColor(128,128,128)][Color(0,128,128,128)][Color(1,154,153,255)][Color(2,153,51,101)][Color(3,255,255,203)]' +
            '[LegendParams(0.2,,1,1,0.75,0.08,,1,0xFFFFFF)]' +
            '[LineWidth(-1,30)]'#0);

          setlength(pX, 0);
          setlength(pY, 0);

        end;

      15 :
        begin
          write('Enter number of nodes : ');
          readln(numZ);
          setlength(pNode, numZ);

          for i := 0 to numZ-1 do begin
            pNode[i].x := -PI + 2*PI * random;
            pNode[i].y := -PI + 2*PI * random;
            pNode[i].z := sin(pNode[i].x) * cos(pNode[i].y);
          end;

          docNum := DPlot_Command(0, '[FileNew(3)]'#0);
          DPlot_AddData8(docNum, DATA_3DR, numZ, 0, pNode[0], pNode[0]);
          setlength(pNode, 0);

        end;

      16 :
// Unit Hemisphere
        begin
          write('step in degrees : ');
          readln(nStep);

          numZ := round((360.0*90.0)/(nStep*nStep) + 1.0);
          setlength(pNode, numZ);

          i := 0;
          lat := 0;
          lon := 0;

          while (lat < 90) do begin
            while (lon < 360) do begin
              pNode[i].x := cos(lon*PI/180.0) * cos(lat*PI/180.0);
              pNode[i].y := sin(lon*PI/180.0) * cos(lat*PI/180.0);
              pNode[i].z := sin(lat*PI/180.0);
              inc(i);
              inc(lon, nStep);
            end;
            inc(lat, nStep);
          end;
          pNode[i].x := 0.0;
          pNode[i].y := 0.0;
          pNode[i].z := 1.0;

// Unit hemisphere
          DPlot.DataFormat := DATA_3DR;
          DPlot.MaxCurves  := 1;
          DPlot.MaxPoints  := numZ;
          DPlot.NumCurves  := 1;
          DPlot.Scale      := SCALE_LINEARX_LINEARY;
          DPlot.NP[0]      := numZ;

          DPlot_Plot8(DPlot, NULL_PTR, pNode[0].x, '[Caption("DPLOTLIB Unit Hemisphere")]'#0);

          setlength(pNode, 0);

        end;

      17 :
// Get a list of all available DPlot DDE topics and display them on the screen.
        begin
          dwSize := sizeof(szTopics);
          retval := DPlot_Request(0, 'Topics', szTopics, dwSize);
          writeln(#13#10+szTopics+#13#10#13#10);
        end;

    end;//case


  until (option = 0);

end.
