c
c FTEST - tests DPLOTLIB.DLL with Compaq Visual Fortran 6.6
c
c This program has been tailored for Compaq Visual Fortran 6.6. Other Fortran compilers may require
c changes, particularly in the interface to DPlot_Plot.
c
c Things to watch for with other compilers and general notes:
c
c 1) By default, most Fortran compilers translate the names of all functions/subroutines to UPPERCASE. 
c    The name "DPlot_Plot" in the DLL is case-specific. In other words you will most likely encounter
c    an "unresolved external" error on DPLOT_PLOT unless you take steps to tell the compiler not to
c    fold all characters to uppercase. If you are certain that you've taken the correct steps in this
c    regard and continue to get linker errors, the problem most likely lies with...
c
c 2) DPLOTLIB.LIB - The import library for DPLOTLIB.DLL. The Compaq Visual Fortran and Absoft Fortran
c    demos use the same import library as the C demo, which was produced as a byproduct of compiling
c    DPLOTLIB.DLL by the MSVC compiler. The DPLOTLIB.LIB used by the WATCOM Fortran demo was produced
c    with "wlib dplotlib +dplotlib.dll". This import library will almost certainly not work with other
c    compilers. If the import library does not work correctly, check your compiler's documentation for
c    the method used to produce import libraries from compiled DLL's.
c
c 3) The last parameter to DPlot_Plot is a character string containing DPlot commands. This character
c    string must be null-terminated (ends in a 0 byte). If you do NOT terminate this character string
c    with a char(0), at best DPlot will report an error in the command string; at worst DPlot and/or 
c    this demo will crash. This restriction does not apply to the character string members of the DPlot 
c    structure, although adding a terminating char(0) to those strings will not hurt anything.
c
      program ftest
      implicit none
      include  'dplot.fi'

      structure /XYZ/
        real*4 x
        real*4 y
        real*4 z
      end structure

      real*4   random

      integer*4 NP, NZ, NX, NY
      parameter (NP=1001, NZ=1000, NX=61, NY=41)

      record  /DPLOT/ DPlot
      real*4  xx, yy
      record  /XYZ/ Node(NZ)
      real*4  x(NP), y(2*NP)
      real*4  extents(4)
      real*4  z(NX*NY)
      real*4  PI
      real*4  dummy
      integer option
      integer i, j, k
      integer ret
      integer seed

      PI = 4.*atan(1.0)

 100  continue

      write(*,*) 'Enter 0 to exit'
      write(*,*) '      1 for XY plot example'
      write(*,*) '      2 for contour plot of gridded data'
      write(*,*) '      3 for contour plot of randomly-spaced points'
      write(*,*) '      4 for box-and-whisker example'
      write(*,*) '? '
      read(*,*) option

      select case(option)
      case(1)
c       Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
c       In this case we've used DATA_XYYY (one X array for one or more Y arrays).
c
c       Since the X values are evenly spaced we could also use
c       DATA_DXY  - X has only 2 elements, DX and X0
c
c       And you can ALWAYS use
c       DATA_XYXY - Each curve uses its own X array. This is the only
c                   option available if the curves have different X values or
c                   a different number of points.
        do i=1,NP
          x(i)   = (4.*i)/(NP-1)
          y(i)   = sin(PI*x(i))
          y(i+NP)= cos(PI*x(i))
        end do

        DPlot.Version     = DPLOT_DDE_VERSION
        DPlot.hwnd        = 0
        DPlot.DataFormat  = DATA_XYYY
        DPlot.MaxCurves   = 2
        DPlot.MaxPoints   = NP
        DPlot.NumCurves   = 2
        DPlot.Scale       = SCALE_LINEARX_LINEARY
        DPlot.LegendX     = 0.05
        DPlot.LegendY	    = 0.05
        DPlot.NP(1)       = NP
        DPlot.NP(2)       = NP
        DPlot.LineType(1) = LINESTYLE_SOLID
        DPlot.LineType(2) = LINESTYLE_LONGDASH
        DPlot.Legend(1)   = 'sin({\sp}x)'
c  	                           {\s is DPlot-speak for "use symbol font"
        DPlot.Legend(2)   = 'cos({\sp}x)'  
        DPlot.Title(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
        DPlot.XAxis       = 'x' 
        DPlot.YAxis       = 'y' 
c		
c       The command string can be as complex as you like, limited to 32K characters.
c       As an alternative, if your program will be producing many similar plots then
c       you might prefer to create a preferences file, edit the file with a text 
c       editor to remove unwanted entries, and use the [GetPreferences("filename")]
c       command. 
c
        ret = DPlot_Plot(loc(DPlot),x(1),y(1),
     1                   '[ManualScale(0,-1.25,4,1.25)]'//
     2                   '[TickInterval(1,0.5,0.25)]'//
     3                   '[Caption("DPLOTLIB XY Test")]'//
     4                   '[DocMaximize()]'//
     5                   '[ClearEditFlag()]'//char(0))

      case(2)
c
c       Contour plot of Z values on a rectangular grid.  z = sin(x)*cos(y)
c
c       extents are, in order, xlo, ylo, xhi, yhi
        extents(1) = -3.
        extents(2) = -2.
        extents(3) =  3.
        extents(4) =  2.
        do i=1,Nx
          xx = extents(1) + ((extents(3)-extents(1))*i)/(Nx-1)
          do j=1,Ny
            yy = extents(2) + ((extents(4)-extents(2))*j)/(Ny-1)
            z((i-1)*Ny+j) = sin(xx)*cos(yy)
          end do
        end do

        DPlot.Version    = DPLOT_DDE_VERSION
        DPlot.hwnd       = 0
        DPlot.DataFormat = DATA_3D
c
c       MaxCurves and MaxPoints members are used for the size of the grid, in grid cells
c
        DPlot.MaxCurves  = Nx-1
        DPlot.MaxPoints  = Ny-1
        DPlot.NumCurves  = 1
        DPlot.Scale      = SCALE_LINEARX_LINEARY
        DPlot.Title(1)   = 'Data sent to DPlot via DPLOTLIB.DLL' 
        DPlot.Title(2)   = 'z = sin(x)*cos(y)'
        DPlot.XAxis      = 'x' 
        DPlot.YAxis      = 'y' 
		
        ret = DPlot_Plot(loc(DPlot),extents(1),z(1),
     1           '[Caption("DPLOTLIB Contour Test 1")]'//
     2           '[Contour3D(1)][ContourGrid(1)][ContourAxes(1)]'//
     3           '[ContourView(30,20)][ContourLevels(20,-1,1)]'//
     4           '[ContourScales(1,1,1)]'//
     5           '[FontPoints(1,10)][FontPoints(6,10)]'//
     6           '[ZAxisLabel("z")][DocMaximize()]'//char(0))

      case(3)
c
c       Contour plot with randomly spaced points
c
        seed = 1
        do i=1,Nz
           Node(i).x = -PI + 2*PI*random(seed)
           Node(i).y = -PI + 2*PI*random(seed)
           Node(i).z = sin(Node(i).x)*cos(Node(i).y)
        end do

        DPlot.Version   = DPLOT_DDE_VERSION
        DPlot.hwnd      = 0
        DPlot.DataFormat= DATA_3DR
        DPlot.MaxCurves = 1
        DPlot.MaxPoints = Nz
        DPlot.NumCurves = 1
        DPlot.Scale     = SCALE_LINEARX_LINEARY
        DPlot.Title(1)  = 'Data sent to DPlot via DPLOTLIB.DLL' 
        DPlot.Title(2)  = 'z = sin(x)*cos(y)'
        DPlot.XAxis     = 'x' 
        DPlot.YAxis     = 'y' 
		
        ret = DPlot_Plot(loc(DPlot),dummy,Node(1).x,
     1           '[Caption("DPLOTLIB Contour Test 2")]'//
     2           '[Contour3D(0)][ContourGrid(0)]'//
     3           '[ContourLevels(21,-1,1)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[DocMaximize()]'//char(0))

      case(4)
c
c       Box-and-whisker plot - 3 groups
c
        DPlot.Version	   = DPLOT_DDE_VERSION
        DPlot.hwnd       = 0
        DPlot.DataFormat = DATA_1D
        DPlot.MaxCurves  = 3
        DPlot.MaxPoints  = 50
        DPlot.NumCurves  = 3
        DPlot.LegendX    = 0.02
        DPlot.LegendY    = 0.7
        DPlot.Scale      = SCALE_LINEARX_LINEARY
        DPlot.Title(1)   = 'Data sent to DPlot via DPLOTLIB.DLL'
        DPlot.XAxis      = ' '
        DPlot.YAxis      = 'Amplitude'

        do i=1,DPlot.NumCurves
          DPlot.NP(i) = DPlot.MaxPoints
          DPlot.Legend(i) = 'Group '//char(64+i)//char(0)
        end do

c       Generate group data, centered around 5.0 with progressively larger variations
        k = 1
        do i=1,DPlot.NumCurves
          do j=1,DPlot.NP(i)
            y(k) = 5.0 + (i+1)*(-0.5+random(seed))
            k = k+1
          end do
        end do
		
        ret = DPlot_Plot(loc(DPlot),dummy,y(1),
     1           '[Caption("DPLOTLIB Box-and-whisker Test")]'//
     2           '[Stat_PlotType(0)][Stat_GroupInfo(15,4)]'//
     3           '[Stat_BWOptions(1,1,0,1,1.5,2,3)]'//
     4           '[Stat_GrandMean(3,30)]'//
     5           '[FontPoints(1,10)][FontPoints(6,10)]'//
     6           '[DocMaximize()]'//char(0))

      case(0)
        goto 9999

      end select

      goto 100
9999  continue
      end
c
c=======================================================================
c
      real*4 function random(iseed)
      implicit none
      integer*4 iseed

      iseed = mod(iseed*7141+54773,259200)
      random = real(iseed)/259200.
      return
      end
