c g77 notes
c
c With the exception of the standard caveats of the switches used to call c
c programs, this program follows the fortran 77 standard.  Use this program
c if the compiler does not understand the VAX structure statements and you 
c cant use fortran 90/95.
c
c One place the program doesnt follow the standard is in the include file, 
c dplot.g77. The DPLOT structure is created using a common block because
c g77 doesn't support /STRUCTURE/'s. The standard doesnt allow mixing of 
c character, reals, and integers in a common block. I dont know of any 
c compilers that dont support this extension, so it shouldnt cause any problems.
c One area where you'll need to pay more attention, though: As a single
c common block is used, all members of that block are preserved. So you'll
c have to take care to initialize more things (to 0 or blank strings or
c whatever) than you would with a local /RECORD/. Otherwise you might get,
c for example, a title line from a previous plot being repeated in a new plot.
c
c There are several problems in this program that may cause crashes or nonsense
c answers. These are as far as I can tell a problem with g77 rather than the code.
c DPlot_Request does NOT work and will either crash or report an insufficient
c buffer size. This is due to passing the incorrect address of the dwSize argument,
c though there doesn't seem to be any reason for that. In my tests, using
c %LOC() or %REF() have not helped. If you figure it out, please let me know at 
c support@dplot.com. With some versions of g77 the intrinsic atan2 function doesn't
c seem to work at all. It crashes this program in one case, and either crashes 
c DPlot with ridiculous data or produces nonsense in the vector plot example, 
c though it DOES work with version 3.10. Again, if you figure it out, let me know.
c
c g77 uses the _cdecl calling convention by default, while all DPLOTLIB.DLL
c functions are _stdcall. There are two workarounds for this:
c
c 1) Using standard _stdcall version of DPLOTLIB.DLL:
c
c    Compile with
c    g77 -c -fno-backslash -fcase-preserve -fno-underscoring -mrtd ftest.f
c    g77 -o ftest ftest.o dplotlib.a
c
c    The downside to doing this occurs if your own program calls functions in
c    some OTHER dll that use the _cdecl calling convention. In that case,
c    you'll need to instead use:
c
c 2) Using _cdecl version of DPLOTLIB.DLL:
c
c    Compile with: 
c
c    g77 -c -fno-backslash -fcase-preserve -fno-underscoring ftest.f
c    g77 -o ftest ftest.o dplotlib_cc.a
c
c    and make sure to include DPLOTLIB_CC.DLL with your project rather than
c    DPLOTLIB.DLL
c
c end g77 notes
c
c FTEST - tests DPLOTLIB.DLL in Fortran.
c
      program ftest
      implicit none
      include  'dplot.g77'
c
      integer option

 100  continue

      write(*,*) 'Enter 0 to exit'
      write(*,*) '      1 XY plot example'
      write(*,*) '      2 Contour plot of gridded data'
      write(*,*) '      3 Contour plot of randomly-spaced points'
      write(*,*) '      4 Box-and-whisker example'
      write(*,*) '      5 Another XY plot (30 curves)'
      write(*,*) '      6 3D scatter plot'
      write(*,*) '      7 Yet another XY plot (world pop. estimate)'
      write(*,*) '      8 Get DPlot/DPlot Jr version number'
      write(*,*) '      9 Get peak values from document 1 '//
     +                    '(DPlot_Request may crash)'
      write(*,*) '     10 Surface plot with concave border'
      write(*,*) '     11 4D surface plot'
      write(*,*) '     12 4D scatter plot'
      write(*,*) '     13 Another contour plot (atan2 crashes)'
      write(*,*) '     14 2D vector plot'
      write(*,*) '     15 Make movie frames of a 3D/4D plot in 3D '//
     +                    'view (DPlot_Request crashes)'
      write(*,*) '     16 Collatz Conjecture (transparent symbols)'


      write(*,*) '? '
      read(*,*) option

      select case(option)
      case(1)
        call XYExample()

      case(2)
        call ContourPlot1()

      case(3)
        call ContourPlot2()

      case(4)
        call BoxAndWhiskerPlot()

      case(5)
        call ThirtyCurves()

      case(6)
        call ThreeDScatter()

      case(7)
        call WorldPopulation()

      case(8)
        call GetVersionNumber()

      case(9)
        call GetPeakValues()

      case(10)
        call ConcaveBorderExample()

      case(11)
        call FourDSurface()

      case(12)
        call FourDScatter()

      case(13)
        call AnotherContourPlot()

      case(14)
        call VectorPlot()

      case(15)
        call MovieFrames()

      case(16)
        call Collatz()

      case(0)
        goto 9999

      end select

      goto 100
9999  continue
      end
c
c=======================================================================
c
      subroutine XYExample()
      implicit none
      include  'dplot.g77'
c
c     Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
c     In this case we've used DATA_XYYY (one X array for one or more Y arrays).
c
c     Since the X values are evenly spaced we could also use
c     DATA_DXY  - X has only 2 elements, DX and X0
c
c     And you can ALWAYS use
c     DATA_XYXY - Each curve uses its own X array. This is the only
c                 option available if the curves have different X values or
c                 a different number of points.

      integer*4 NP
      parameter (NP=1001)
      integer*4 i
      integer*4 ret
      real*8    x(NP), y(2*NP)
      real*8    PI

      integer  DPlot_Plot8
      external DPlot_Plot8

      ret = 1
      PI = 4.*atan(1.0)
      do i=1,NP
        x(i)   = (4.*i)/(NP-1)
        y(i)   = sin(PI*x(i))
        y(i+NP)= cos(PI*x(i))
      end do

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_XYYY
      DPMaxCurves   = 2
      DPMaxPoints   = NP
      DPNumCurves   = 2
      DPScale       = SCALE_LINEARX_LINEARY
      DPLegendX     = 0.05
      DPLegendY     = 0.05
      DPNP(1)       = NP
      DPNP(2)       = NP
      DPLineType(1) = LINESTYLE_SOLID
      DPLineType(2) = LINESTYLE_LONGDASH
      DPLegend(1)   = 'sin({\sp}x)'
c                              {\s is DPlot-speak for "use symbol font"
      DPLegend(2)   = 'cos({\sp}x)'  
      DPTitle(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPTitle(2)    = ' ' 
      DPTitle(3)    = ' ' 
      DPXAxis       = 'x' 
      DPYAxis       = 'y' 
c		
c     The command string can be as complex as you like, limited to 32K characters.
c     As an alternative, if your program will be producing many similar plots then
c     you might prefer to create a preferences file, edit the file with a text 
c     editor to remove unwanted entries, and use the [GetPreferences("filename")]
c     command. 
c
      ret = DPlot_Plot8(DPVersion,x,y,
     +                 '[ManualScale(0,-1.25,4,1.25)]'//
     +                 '[TickInterval(1,0.5,0.25)]'//
     +                 '[LineWidth(-2,30)]'//            ! Set all line widths to 30/1000ths of an inch
     +                 '[Caption("DPLOTLIB XY Test")]'//
     +                 '[DocMaximize()]'//
     +                 '[ClearEditFlag()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine ContourPlot1()
      implicit none
      include  'dplot.g77'
c
c     Contour plot of Z values on a rectangular grid.  z = sin(x)*cos(y)
c
      integer*4 NX, NY
      parameter (NX=61, NY=41)
      integer*4 i, j, ret
      real*8    extents(4)
      real*8    xx, yy
      real*8    z(NX*NY)

      integer  DPlot_Plot8
      external DPlot_Plot8

c     extents are, in order, xlo, ylo, xhi, yhi
      extents(1) = -3.
      extents(2) = -2.
      extents(3) =  3.
      extents(4) =  2.
      do i=1,NX
        xx = extents(1) + ((extents(3)-extents(1))*i)/(NX-1)
        do j=1,NY
          yy = extents(2) + ((extents(4)-extents(2))*j)/(NY-1)
          z((i-1)*NY+j) = sin(xx)*cos(yy)
        end do
      end do

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_3D
c
c     MaxCurves and MaxPoints members are used for the size of the grid, in grid cells
c
      DPMaxCurves   = NX-1
      DPMaxPoints   = NY-1
      DPNumCurves   = 1
      DPScale       = SCALE_LINEARX_LINEARY
      DPTitle(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPTitle(2)    = 'z = sin(x)*cos(y)'
      DPTitle(3)    = ' '
      DPXAxis       = 'x' 
      DPYAxis       = 'y' 
		
      ret = DPlot_Plot8(DPVersion,extents,z,
     1       '[Caption("DPLOTLIB Contour Test 1")]'//
     2       '[Contour3D(1)][ContourGrid(1)][ContourAxes(1)]'//
     3       '[ContourView(30,20)][ContourLevels(20,-1,1)]'//
     4       '[ContourScales(1,1,1)]'//
     5       '[ContourLighting(2,0.2)]'//                     ! Phong shading, 20% ambient light
     6       '[FontPoints(1,10)][FontPoints(6,10)]'//
     7       '[FontPoints(4,12)][FontPoints(5,12)][FontPoints(8,12)]'//
     8       '[ZAxisLabel("z")][DocMaximize()]'//char(0))
      return
	  end
c
c======================================================================
c
      subroutine ContourPlot2()
      implicit none
      include  'dplot.g77'
c
c     Contour plot with randomly spaced points
c
      integer*4 i, ret
      real*8    PI
      real*8    dummy

      integer*4 NZ
      parameter (NZ=1000)
      real*8 Node(3,NZ)

      integer  DPlot_Plot8
      external DPlot_Plot8

      PI = 4.*atan(1.0)
      do i=1,NZ
         Node(1,i) = -PI + 2*PI*Rand(0)
         Node(2,i) = -PI + 2*PI*Rand(0)
         Node(3,i) = sin(Node(1,i))*cos(Node(2,i))
      end do

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_3DR
      DPMaxCurves   = 1
      DPMaxPoints   = NZ
      DPNumCurves   = 1
      DPScale       = SCALE_LINEARX_LINEARY
      DPTitle(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPTitle(2)    = 'z = sin(x)*cos(y)'
      DPTitle(3)    = ' '
      DPXAxis       = 'x' 
      DPYAxis       = 'y' 
		
      ret = DPlot_Plot8(DPVersion,dummy,Node,
     1           '[Caption("DPLOTLIB Contour Test 2")]'//
     2           '[Contour3D(0)][ContourGrid(0)]'//
     3           '[ContourLevels(21,-1,1)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[DocMaximize()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine BoxAndWhiskerPlot()
      implicit none
      include  'dplot.g77'
c
c     Box-and-whisker plot - 3 groups
c
      integer*4 i, j, k, ret
      real*8    dummy

      integer*4 NP
      parameter (NP=150)
      real*8  y(NP)

      integer  DPlot_Plot8
      external DPlot_Plot8

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_1D
      DPMaxCurves   = 3
      DPMaxPoints   = 50
      DPNumCurves   = 3
      DPLegendX		= 0.02
      DPLegendY		= 0.7
      DPScale       = SCALE_LINEARX_LINEARY
      DPTitle(1)    = 'Data sent to DPlot via DPLOTLIB.DLL'
      DPTitle(2)    = ' ' 
      DPTitle(3)    = ' ' 
      DPXAxis       = ' '
      DPYAxis       = 'Amplitude'

      do i=1,DPNumCurves
        DPNP(i) = DPMaxPoints
        DPLegend(i) = 'Group '//char(64+i)//char(0)
      end do

c     Generate group data, centered around 5.0 with progressively larger variations
      k = 1
      do i=1,DPNumCurves
        do j=1,DPNP(i)
          y(k) = 5.0 + (i+1)*(-0.5+Rand(0))
          k = k+1
        end do
      end do
		
      ret = DPlot_Plot8(DPVersion,dummy,y,
     1           '[Caption("DPLOTLIB Box-and-whisker Test")]'//
     2           '[Stat_PlotType(0)][Stat_GroupInfo(15,4)]'//
     3           '[Stat_BWOptions(1,1,0,1,1.5,2,3)]'//
     4           '[Stat_GrandMean(3,30)]'//
     5           '[FontPoints(1,10)][FontPoints(6,10)]'//
     6           '[DocMaximize()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine ThirtyCurves()
      implicit none
      include 'dplot.g77'

      integer NC, NP
      parameter (NC=30, NP=1001)
      real*8 x(NP), y(NP*NC)
      integer i, j, ret
      real*8  PI
      parameter (PI=3.1415926535897932384626433832795)
      integer  DPlot_Plot8
      external DPlot_Plot8

c     Simulate a square wave with a slew of sine waves. First 29 curves are
c     sine waves, last curve is the addition of 1st 29.

      do i=0,NP-1
        x(i+1) = (2.*PI*i)/(NP-1)
        y(i+(NC-1)*NP+1) = 0.0
        do j=0,NC-2
          y(i+j*NP+1) = sin((j*2+1)*x(i+1))/(j*2+1)
          y(i+(NC-1)*NP+1) = y(i+(NC-1)*NP+1) + y(i+j*NP+1)
        end do
        y(i+(NC-1)*NP+1) = y(i+(NC-1)*NP+1) * 4.0/PI
      end do

      DPVersion		    = DPLOT_DDE_VERSION
      DPhwnd			= 0
      DPDataFormat	    = DATA_XYYY
      DPMaxCurves		= NC
      DPMaxPoints		= NP
      DPNumCurves		= NC
      DPScale			= SCALE_LINEARX_LINEARY
      do j=1,NC
        DPNP(j) = NP
        DPLineType(j) = LINESTYLE_SOLID
        if(j .lt. NC) write(DPLegend(j),'(A,I2,A,I2)')
     +     'sin(',j*2-1,'x)/',j*2-1
      end do
      DPLegend(NC) = '4/{\sp}(sin(x)+sin(3x)/3+sin(5x)/5+...)'
      DPTitle(1) = 'Square wave approximation'
      DPTitle(2) = ' ' 
      DPTitle(3) = ' ' 
      DPXAxis    = 'x'
      DPYAxis    = 'y'

      ret = DPlot_Plot8(DPVersion,x,y,
     +       '[ManualScale(0,-1.25,6.5,1.25)]'//
     +       '[TickInterval(1,0.5,0.25)]'//
     +       '[LegendParams(0.25,10,1,1,0,1,0,8)]'//
     +       '[LineWidth(-2,0)][Caption("DPLOTLIB XY Test")]'//
     +       '[DocMaximize()][ClearEditFlag()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine ThreeDScatter()
      implicit none
      include  'dplot.g77'
c
c     3D scatter plot
c
      integer*4 i, j, k
      integer*4 DocNum
      real*8    xmin, xmax, ymin, ymax, zmin, zmax
      real*8    xcenter, ycenter, zcenter
      real*8    phi, theta, theta_inc, radius
      real*8    dummy
      real*8    PI
      character szCommands*1024
      real*8    Node(3,3000)

      integer  DPlot_Plot8
      external DPlot_Plot8

      PI = 4.*atan(1.0)

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_3DS
      DPMaxCurves   = 3
      DPMaxPoints   = 1500
      DPNumCurves   = 3
      DPScale       = SCALE_LINEARX_LINEARY
      DPLegend(0)   = char(0)
      DPLegend(1)   = char(0)
      DPLegend(2)   = char(0)
      DPLegend(3)   = char(0)
      DPTitle(1)    = 'DPLOTLIB 3D Scatter Plot'
      DPTitle(2)    = ' ' 
      DPTitle(3)    = ' ' 
      DPXAxis       = 'x' 
      DPYAxis       = 'y' 

      DPNP(1) = 500
      DPNP(2) = 1000
      DPNP(3) = 1500
c
c     Keep track of the minimum and maximum X,Y, and Z values so we can
c     force plot extents to good values, "good" in this case being
c     whole numbers.
c
      xmin = 0
      xmax = 0
      ymin = 0
      ymax = 0
      zmin = 0
      zmax = 0

      k = 1       ! index into Scatter array
      do i = 1, DPNumCurves
c
c       Center of each sphere is at some random location between
c       (-5,-5,-5) and (5,5,5).
c
        xcenter = -5.0 + 10.0*Rand(0)
        ycenter = -5.0 + 10.0*Rand(0)
        zcenter = -5.0 + 10.0*Rand(0)
c
c       radii are proportional to number of points
c
        radius = 5.D-3 * DPNP(i)
c
        xmin = dmin1(xmin,xcenter-radius)
        xmax = dmax1(xmax,xcenter+radius)
        ymin = dmin1(ymin,ycenter-radius)
        ymax = dmax1(ymax,ycenter+radius)
        zmin = dmin1(zmin,zcenter-radius)
        zmax = dmax1(zmax,zcenter+radius)
c
c       increment in theta (angle around z axis) gives roughly
c       equal spacing between points in all directions:
c
        theta_inc = 2.0 * sqrt(dble(DPNP(i)))
        do j = 1, DPNP(i)
          phi = -PI/2 + (PI * dble(j-1) / dble(DPNP(i)))
          theta = theta_inc * PI * dble(j-1) / dble(DPNP(i))
          Node(1,k) = xcenter + (radius * cos(theta) * cos(phi))
          Node(2,k) = ycenter + (radius * sin(theta) * cos(phi))
          Node(3,k) = zcenter + (radius * sin(phi))
          k = k + 1
        end do
      end do
c
c g77 for some reason doesn't like int(xmin-0.5), etc. No idea why,
c as this works as expected with other compilers. If you have a clue,
c LET ME KNOW at support@dplot.com
c
c     write(szCommands,'(A,I3,1H,,I3,1H,,I3,1H,,I3,1H,,I3,1H,,I3,A)')
c    1           '[Caption("DPLOTLIB 3D Scatter Plot")]'//
c    2           '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
c    3           '[TickInterval(1,1,1,1)][ContourAxes(1)]'//
c    4           '[ColorBasis(1)][ContourColorScheme(2)]'//
c    5           '[ContourCustomColors(255,65280,16711680)]'//
c    6           '[ContourScales(1,1,1)][SymbolOutline(1)]'//
c    7           '[ZAxisLabel("z")]'//
c    8           '[ManualScale(',
c    9           int(xmin-0.5),int(ymin-0.5),
c    A           int(xmax+0.5),int(ymax+0.5),
c    B           int(zmin-0.5),int(zmax+0.5),
c    C           ')][FontPoints(1,10)][FontPoints(6,10)]'//
c    D           '[DocMaximize()]'//char(0)

      szCommands = 
     +           '[Caption("DPLOTLIB 3D Scatter Plot")]'//
     +           '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
     +           '[TickInterval(1,1,1,1)][ContourAxes(1)]'//
     +           '[ColorBasis(1)][ContourColorScheme(2)]'//
     +           '[ContourCustomColors(255,65280,16711680)]'//
     +           '[ContourScales(1,1,1)][SymbolOutline(1)]'//
     +           '[ZAxisLabel("z")]'//
     +           '[FontPoints(1,10)][FontPoints(6,10)]'//
     +           '[DocMaximize()]'//char(0)

      DocNum = DPlot_Plot8(DPVersion,dummy,Node,szCommands)
      return
      end
c
c======================================================================
c
      integer function ExcelDay(Month,Day,Year)
      implicit none
      integer*2 Month, Day, Year
      
	if(Year .lt. 1900 .or. Year .gt. 2078) then
        ExcelDay = -1
      else
        if(Month .gt. 2) then
          ExcelDay = int(365.25*Year)+int(30.6001*(Month+1))+Day-694037
        else
          ExcelDay = int(365.25*(Year-1))+int(30.6001*(Month+13))+Day-
     +                694037
        endif
      endif
      return
      end
c
c======================================================================
c
      subroutine WorldPopulation()
      implicit none
      include 'dplot.g77'
c
c     Estimate of world population
c
      integer NUMDAYS, SAMPLESPERDAY
      parameter (NUMDAYS=10,SAMPLESPERDAY=24*60*6)    ! 10 second intervals
      real*8 REPLACEMENTRATE
      parameter (REPLACEMENTRATE=0.013)

      integer*2 Year, Month, Day
      integer*2 Hrs, Mins, Secs, HSecs
      integer NP
      parameter (NP=NUMDAYS*SAMPLESPERDAY+1)
      real*8  JanOne04Pop
      parameter (JanOne04Pop = 6393657063.)
      real*8  BaseDay
      real*8  CurrentDate
      real*8  elapsed_time
      real*8  x(NP), y(NP)
      integer i, ret
      character Clock(3)*12
      integer Values(8)

      integer  ExcelDay
      external ExcelDay
      integer  DPlot_Plot8
      external DPlot_Plot8

      Month = 1
      Day   = 1
      Year  = 2004
      BaseDay = ExcelDay(Month,Day,Year)
c
      call Date_and_Time(Clock(1),Clock(2),Clock(3),Values)

      Year  = Values(1)
      Month = Values(2)
      Day   = Values(3)
      CurrentDate = dble(ExcelDay(Month,Day,Year)) +
     +         dble(Values(5))/24 + dble(Values(6))/(24*60)

c     We'll estimate the world population every 30 seconds for NUMDAYS days
      
      do i=1,NP
        x(i) = CurrentDate + dble(i-1)/dble(SAMPLESPERDAY)
        elapsed_time = x(i) - BaseDay
        elapsed_time = x(i) - BaseDay
c g77 doesn't like this for some reason (crashes).
c       y(i) = dint( JanOne04Pop * 
c    +          exp(REPLACEMENTRATE*elapsed_time/365.25))
c for example, this works:
c       y(i) = Rand(0)
c what about getting rid of dint?
        y(i) = JanOne04Pop * 
     +         exp(REPLACEMENTRATE*elapsed_time/365.25)
c Yep! That works, but I have no idea why!
      end do

      DPVersion		= DPLOT_DDE_VERSION
      DPhwnd          = 0
      DPDataFormat	= DATA_XYXY
      DPMaxCurves		= 1
      DPMaxPoints		= NP
      DPNumCurves		= 1
      DPScale			= SCALE_LINEARX_LINEARY
      DPNP(1)			= NP
      DPLineType(1)	= LINESTYLE_SOLID
      DPLegend(0)     = char(0)
      DPLegend(1)     = char(0)
      DPTitle(1) = 'World Population Estimate'
      write(DPTitle(2),'(A,F11.0)')
     +  'based on Jan 1, 2004 population of ',JanOne04Pop
      write(DPTitle(3),'(A,F4.1,1H%)') 'and a replacement rate of ',
     +        100*REPLACEMENTRATE
      DPXAxis = 'Date'
      DPYAxis = 'Population'

      ret = DPlot_Plot8(DPVersion,x,y,
     +       '[Caption("DPLOTLIB XY Test")]'//
     +       '[DateFormat("M/dd/yy")]'//
     +       '[TimeFormat("h:mm:ss")]'//
     +       '[NumberFormat(0,6)][NumberFormat(1,11)]'//
     +       '[ClearEditFlag()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine GetVersionNumber()
      implicit none
      include  'dplot.g77'

      integer result
      character version*32
      integer L
      integer  DPlot_GetVersion
      external DPlot_GetVersion

      result = DPlot_GetVersion(version)
      if(result .le. 0) then
        write(*,*) 'DPlot_GetVersion failed!'
      else
        L = index(version,char(0))-1
        if(L .le. 0) L = len(version)
        if(result .eq. 1) then
          write(*,*) 'DPlot Version ',version(1:L)
        else
          write(*,*) 'DPlot Jr Version ',version(1:L)
        endif
      endif
      return
      end
c
c======================================================================
c
      subroutine GetPeakValues()
      implicit none
      include  'dplot.g77'
c
c     Get min/max and other values from document 1. The returned
c     string is equivalent to what you get with the List Peaks
c     menu command in DPlot.
c
      character szPeaks*1024
      integer   dwSize
      integer   ret

      integer  DPlot_Request
      external DPlot_Request

c
c DPlot_Request may bomb with g77, or it may simply return an 
c "insufficient buffer size" error without changing dwSize.
c It doesn't seem to like the dwSize argument being passed 
c by reference (which is correct!). Using %LOC or %REF have
c no effect (which would be expected since arguments are passed
c by reference by default). I've no idea why this happens. If you
c figure it out, please let me know at support@dplot.com
c
      dwSize = len(szPeaks)
      ret = DPlot_Request(%VAL(1),'ListPeaks'//char(0),szPeaks,
     +                    dwSize)
      if(ret .gt. 0) then
        dwSize=index(szPeaks,char(0))-1
        if(dwSize .le. 0) dwSize = len(szPeaks)
        write(*,*) szPeaks(1:dwSize)
      elseif(ret .eq. -3) then
        write(*,*) 'Insufficient buffer size, ',dwSize,' required.'
      else
        write(*,*) 'Return value of DPlot_Request=',ret
      endif
      return
      end
c
c======================================================================
c
      subroutine ConcaveBorderExample()
      implicit none
      include  'dplot.g77'
c
c     Surface plot with concave border
c
      integer*4 i, DocNum, ret
      real*8    x0, y0, side
      real*8    dummy
      real*8    PI
      real*8    Node(3,10)
      real*8    Border(2,10)
      character stuff*2

      integer  DPlot_3DBorder
      external DPlot_3DBorder
      integer  DPlot_Plot8
      external DPlot_Plot8

      PI = 4.*atan(1.0)

      x0 = 0.             ! center of star/pentagon
      y0 = 0.
      side = 1.0          ! length of each pentagon side

      Node(1,1) = x0 - 0.5*side
      Node(2,1) = y0 - 0.5*side*tan(54*PI/180)
      Node(3,1) = 0.0
      Node(1,2) = x0
      Node(2,2) = Node(2,1)+0.5*side*tan(36*PI/180)
      Node(3,2) = 1.0
      Node(1,3) = x0 + 0.5*side
      Node(2,3) = Node(2,1)
      Node(3,3) = 0.0
      Node(1,4) = Node(1,3)-0.5*side*(sin(18*PI/180)/cos(36*PI/180))
      Node(2,4) = Node(2,3)+0.5*side*(cos(18*PI/180)/cos(36*PI/180))
      Node(3,4) = 1.0
      Node(1,5) = Node(1,3)+side*sin(18*PI/180)
      Node(2,5) = Node(2,3)+side*cos(18*PI/180)
      Node(3,5) = 0.0
      Node(1,6) = x0+0.5*side*(sin(18*PI/180)/cos(36*PI/180))
      Node(2,6) = Node(2,5)
      Node(3,6) = 1.0
      Node(1,7) = x0
      Node(2,7) = Node(2,5)+side*sin(36*PI/180)
      Node(3,7) = 0.0
      Node(1,8) = 2*x0 - Node(1,6)
      Node(2,8) = Node(2,6)
      Node(3,8) = 1.0
      Node(1,9) = 2*x0 - Node(1,5)
      Node(2,9) = Node(2,5)
      Node(3,9) = 0.0
      Node(1,10) = 2*x0 - Node(1,4)
      Node(2,10) = Node(2,4)
      Node(3,10) = 1.0

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_3DR
      DPMaxCurves   = 1
      DPMaxPoints   = 10
      DPNumCurves   = 1
      DPScale       = SCALE_LINEARX_LINEARY
      DPTitle(1)    = 'Surface Plot with Concave Edges'
      DPTitle(2)    = 'See {\iDefine Boundary} command '//
     1                    'on {\iOptions} menu'
      DPTitle(3)    = ' ' 

      DPXAxis       = 'x' 
      DPYAxis       = 'y' 

      DocNum = DPlot_Plot8(DPVersion,dummy,Node,
     1           '[Caption("DPLOTLIB Surface Plot w/ Concave Edges")]'//
     2           '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
     3           '[ContourLevels(20,0,1)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[DocMaximize()]'//char(0))
c
c     At this point surface is convex - a pentagon. Add border
c     points, which in this example are simply a repeat of the x,y points
c     of the surface
c
      if(DocNum .gt. 0) then
        do i=1,10
          Border(1,i) = Node(1,i)
          Border(2,i) = Node(2,i)
        end do
        ret = DPlot_3DBorder(%VAL(DocNum),%VAL(10),Border)
      endif
      return
      end
c
c=======================================================================
c
      integer function floor(x)
      implicit none
      real*8 x

      if(x .ge. 0) then
        floor = int(x)
      elseif(dmod(x,1.D0) .eq. 0.0) then
        floor = int(x)
      else
        floor = int(x)-1
      endif
      return
      end
c
c=======================================================================
c
      integer function ceil(x)
      implicit none
      real*8 x

      if(x .le. 0) then
        ceil = int(x)
      elseif(dmod(x,1.D0) .eq. 0.0) then
        ceil = int(x)
      else
        ceil = int(x)+1
      endif
      return
      end
c
c=======================================================================
c
      integer function lentrim(s)
      implicit none
      character s*(*)

      do lentrim = len(s), 1, -1
        if (s(lentrim:lentrim) .ne. ' ') return
      end do
      return
      end
c
c=======================================================================
c
      subroutine FourDSurface()
      implicit none
      include 'dplot.g77'
c
c     4D surface plot on a rectangular grid
c
      integer Nx, Ny            ! Rows and columns
      parameter (Nx=121, Ny=121)
      integer i, j, k, ret
      real*8  x, y, z
      real*8  dummy
      real*8 Node(4,Nx*Ny)

      integer  DPlot_Plot8
      external DPlot_Plot8

      k = 1
      do i = 0,Nx-1
        x = 5.0 + 30.*dble(i)/dble(Nx-1)
        do j = 0,Ny-1
          Node(1,k) = x
          y = 5.0 + 30.*dble(j)/dble(Ny-1)
          Node(2,k) = y
          if(x .eq. 20. .and. y .eq. 20.) then
            z = sin(1.0)
          else
            z = sin(sqrt((x-20.)*(x-20.)+(y-20.)*(y-20)))/
     +          sqrt((x-20.)*(x-20.)+(y-20.)*(y-20.))
          endif
          z = (z + 1.5)*100.
          Node(3,k) = z
          Node(4,k) = 10. * (1.1 + sin((x-20.)/5.)*cos((y-20.)/10.))
          k = k+1
        end do
      end do

      DPVersion		= DPLOT_DDE_VERSION
      DPDataFormat	= DATA_4D
      DPMaxCurves		= 1
      DPMaxPoints		= Nx*Ny
      DPNumCurves		= 1
      DPScale			= SCALE_LINEARX_LINEARY
      DPTitle(1) = 'Data sent to DPlot via DPLOTLIB.DLL'
      DPTitle(2) = 'z = 100 * (sin(sqrt((x-20.){\u2}+'//
     +                                     '(y-20.){\u2})) / '//
     +                      'sqrt((x-20.){\u2}+(y-20.){\u2}) + 1.5)'
      DPTitle(3) = 'a = 10. * (1.1 + sin((x-20.)/5.)*'//
     +                 'cos((y-20.)/10.))'
      DPXAxis    = 'x'
      DPYAxis    = 'y'
	
      ret = DPlot_Plot8(DPVersion,dummy,Node,
     +   '[Caption("DPLOTLIB 4D Surface Plot Test")]'//
     +   '[Contour3D(1)][ContourMethod(0)][ContourGrid(0)]'//
     +   '[ContourAxes(1)][ContourView(118,28)]'//
     +   '[ContourLevels(22,0,22)][ContourScales(1,1,0.2)]'//
     +   '[ContourLighting(0,0.05)]'//
     +   '[ContourColorScheme(2)]'//
     +   '[ContourCustomColors(0,0x00FF047D,0x000B1EB3,0x000068DC,'//
     +   '0x0000FAFE)]'//
     +   '[FontPoints(1,10)][FontPoints(4,14)][FontPoints(5,14)]'//
     +   '[FontPoints(6,10)][FontPoints(8,14)]'//
     +   '[ZAxisLabel("z")][AAxisLabel("Amplitude")][DocMaximize()]'//
     +   char(0))
      return
      end
c
c=======================================================================
c
      subroutine FourDScatter()
      implicit none
      include 'dplot.g77'
c
c     4D scatter plot - random points on 3 spheres.
c
      integer NC, Nz
      parameter (NC=3, Nz=1000)
c
c     Nz is the number of points in each of the NC data sets. Data sets CAN have
c     differing numbers of points. In either case, the Node array is packed with 
c     no unused nodes. For example the first point in the 2nd data set should
c     immediately follow the last point in the 1st data set.
c
      integer i, j, ret
      real*8 zz, radius, theta, phi
      real*8 theta_inc
      real*8 xmin, xmax, ymin, ymax, zmin, zmax
      real*8 xx, yy
      real*8 dummy
      real*8  PI
      parameter (PI=3.1415926535897932384626433832795)
      real*8 Node(4,NC*Nz)
      character cmd*512

      integer  ceil, floor
      external ceil, floor
      integer  DPlot_Plot8
      external DPlot_Plot8

      xmin = 0
      xmax = 0
      ymin = 0
      ymax = 0
      theta_inc = 2.0*sqrt(dble(Nz))
      do j=0,NC-1
        xx = -10.+20.*Rand(0)         ! Center of sphere
        yy = -10.+20.*Rand(0)
        zz = -10.+20.*Rand(0)
        radius = 4.+4.*Rand(0)
        xmin = dmin1(xmin,xx-radius)
        xmax = dmax1(xmax,xx+radius)
        ymin = dmin1(ymin,yy-radius)
        ymax = dmax1(ymax,yy+radius)
        zmin = dmin1(zmin,zz-radius)
        zmax = dmax1(zmax,zz+radius)
        do i=j*Nz,(j+1)*Nz-1
          phi = -PI/2+PI*dble(i-j*Nz)/dble(Nz)
          theta = theta_inc*PI*dble(i-j*Nz)/dble(Nz)
          Node(1,i+1) = xx + radius*cos(theta)*cos(phi)
          Node(2,i+1) = yy + radius*sin(theta)*cos(phi)
          Node(3,i+1) = zz + radius*sin(phi)
          Node(4,i+1) = Node(1,i+1)               ! No reason, but it has to be something!
        end do
        write(DPLegend(j+1),'(A,I1)') 'Sphere ',j+1
        DPNP(j+1) = Nz
      end do

      DPVersion           = DPLOT_DDE_VERSION
      DPhwnd              = 0
      DPDataFormat        = DATA_4DS
      DPMaxCurves         = NC
      DPMaxPoints         = Nz
      DPNumCurves         = NC
      DPScale             = SCALE_LINEARX_LINEARY
      DPLegend(0)         = 'Legend'
      DPTitle(1)          = 'DPLOTLIB - Points on Spheres'
      DPTitle(2)          = 'Amplitude=X'
      DPTitle(3)          = ' '
      DPXAxis             = 'x'
      DPYAxis             = 'y'
c
c g77 doesn't like my replacement intrinsics. Don't know why.
c
c     xmin = floor(xmin)
c     xmax = ceil(xmax)
c     ymin = floor(ymin)
c     ymax = ceil(ymax)
c     zmin = floor(zmin)
c     zmax = ceil(zmax)

c g77 doesn't like the formatting for the ManualScale command,
c though I can't figure out why. If you know please share at
c support@dplot.com
c
c     write(cmd,'(A,5(F4.0,1H,),F4.0,A,A)')
c    +  '[Caption("DPLOTLIB Test 12")]'//
c    +  '[TickInterval(1,1,1,1)]'//
c    +  '[ManualScale(',xmin,ymin,xmax,ymax,zmin,zmax,
c    +  ')][ColorBasis(0)][ContourColorScheme(2)]'//
c    +  '[ContourCustomColors(255,65535,16711680)]'//
c    +  '[ContourScales(1,1,1)]'//
c    +  '[FontPoints(1,10)][FontPoints(6,10)]'//
c    +  '[DocMaximize()][SymbolType(-1,288)]'//
c    +  '[SymbolSize(1,100)][SymbolSize(2,150)][SymbolSize(3,200)]'//
c    +  '[SymbolOutline(1)]',
c    +  char(0)

      cmd = 
     +  '[Caption("DPLOTLIB Test 12")]'//
     +  '[TickInterval(1,1,1,1)]'//
     +  '[ColorBasis(0)][ContourColorScheme(2)]'//
     +  '[ContourCustomColors(255,65535,16711680)]'//
     +  '[ContourScales(1,1,1)]'//
     +  '[FontPoints(1,10)][FontPoints(6,10)]'//
     +  '[DocMaximize()][SymbolType(-1,288)]'//
     +  '[SymbolSize(1,100)][SymbolSize(2,150)][SymbolSize(3,200)]'//
     +  '[SymbolOutline(1)]'//char(0)

      ret = DPlot_Plot8(DPVersion,dummy,Node,cmd)
      return
      end
c
c=======================================================================
c
      subroutine AnotherContourPlot()
      implicit none
      include 'dplot.g77'

      integer Nx, Ny
      parameter (Nx=361,Ny=161)
      integer i, j, ret
      real*8  extents(4)
      real*8  xx, yy
      real*8  z(Nx*Ny)
      real*8  Vc, Vs, Vnet, Vf
      real*8  theta
      real*8  PI
      parameter (PI=3.1415926535897932384626433832795)

      integer  DPlot_Plot8
      external DPlot_Plot8

      extents(1) = 0.0       ! xlo
      extents(2) = 0.0       ! ylo
      extents(3) = 360.0     ! xhi
      extents(4) = 1.0       ! yhi

      do i=1,Nx
        xx = extents(1) + ((extents(3)-extents(1))*(i-1))/(Nx-1)
        do j=1,Ny
          yy = extents(2) + ((extents(4)-extents(2))*(j-1))/(Ny-1)
          Vc = 1.0-yy*cos(xx*PI/1.8D2)
          Vs = yy*sin(xx*PI/1.8D2)
          Vnet = sqrt(Vc*Vc + Vs*Vs)
c
c atan2 crashes under some (as yet) unknown circumstances with g77,
c though it works with every other FORTRAN compiler.
c Having both arguments to atan2 = 0 on Salford FORTRAN causes
c a problem that is fixed by the if block below, but that has no
c effect here. If you know why, please let me know at 
c support@dplot.com
c
          if(Vc .eq. 0.0 .and. Vs .eq. 0.0) then
            theta = 0.0
          else
            theta = atan2(Vs*Vs,Vc*Vc)
          endif
          Vf = Vnet*Vnet * abs(cos(theta))
          if(Vc .lt. 0) then
            z((i-1)*Ny+j) = -Vf
          else
            z((i-1)*Ny+j) = Vf
          endif
        end do
      end do

      DPVersion         = DPLOT_DDE_VERSION
      DPhwnd            = 0
      DPDataFormat      = DATA_3D
      DPMaxCurves       = Nx-1               ! Size of the grid, in grid cells
      DPMaxPoints       = Ny-1
      DPNumCurves       = 1
      DPScale           = SCALE_LINEARX_LINEARY
      DPTitle(1) = 'Relative Drag Force for a Spherical Object'
      DPTitle(2) = 'Drag{\dactual}/Drag{\dno wind}\n'
      DPTitle(3) = ' ' 
      DPXAxis    = 'Wind Direction, degrees'
      DPYAxis    = 'Wind Speed/Ground Speed'

      ret = DPlot_Plot8(DPVersion,extents,z,char(0))

      return

      ret = DPlot_Plot8(DPVersion,extents,z,
     + '[Caption("DPLOTLIB Contour Test 13")]'//
     + '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
     + '[ContourAxes(1)][ContourLevels(40,0,4)]'//
     + '[ContourScales(1,360,1)][ContourLighting(2,0.05)]'//
     + '[ManualScale(0,0,360,1,0,4)][TickInterval(1,10,0.1,1)]'//
     + '[FontPoints(1,10)][FontPoints(6,10)][ZAxisLabel("z")]'//
     + '[XYLabel(0,1,0,"Tailwind")][XYLabel(180,1,0,"Headwind")]'//
     + '[XYLabel(360,1,0,"Tailwind")][DocMaximize()][ClearEditFlag()]'//
     +  char(0))

      return
      end
c
c=======================================================================
c
      subroutine VectorPlot()
      implicit none
      include  'dplot.g77'

      integer Nx, Ny, NP
      parameter (Nx=37, Ny=16, NP=Nx*Ny)  ! Columns, rows, total points/curve
      integer i, j, k, ret
      real*8  xlo, ylo, xhi, yhi
      real*8  xx, yy
      real*8  x(NP), y(3*NP)
      real*8  alpha

      integer  DPlot_Plot8
      external DPlot_Plot8

      real*8  PI
      parameter (PI=3.1415926535897932384626433832795)

      xlo = 0.0
      ylo = 0.0
      xhi = 360.0
      yhi = 1.5
      k = 1
      do i=1,Nx
        xx = xlo + ((xhi-xlo)*(i-1))/(Nx-1)
        do j = 1,Ny
          yy = ylo + ((yhi-ylo)*(j-1))/(Ny-1)
          x(k) = xx
          y(k) = yy
          alpha = xx
          if(alpha .lt. 0) alpha = alpha+360
c
c atan2 doesn't work with g77. If you know why, please let
c me know at support@dplot.com
c
          y(k+NP) = PI-atan2(yy*dsin(alpha*PI/1.8D2),
     +                       1.D0-yy*dcos(alpha*PI/1.8D2))
          if(y(k+NP) .lt. 0.0) y(k+NP) = y(k+NP) + 2.*PI
          if(y(k+NP) .ge. 2.*PI) y(k+NP) = y(k+NP) - 2.*PI
          y(k+2*NP) = sqrt(1.0-2*yy*dcos(alpha*PI/1.8D2)+yy*yy)
          k = k+1
        end do
      end do

      DPVersion       = DPLOT_DDE_VERSION
      DPhwnd          = 0
      DPDataFormat    = DATA_XYYY
      DPMaxCurves     = 3
      DPMaxPoints     = NP
      DPNumCurves     = 3
      DPScale         = SCALE_LINEARX_LINEARY
      DPNP(1)         = NP
      DPNP(2)         = NP
      DPNP(3)         = NP
      DPLineType(1)   = LINESTYLE_NONE
      DPLineType(2)   = LINESTYLE_NONE
      DPLineType(3)   = LINESTYLE_NONE
      DPSymbolType(1) = SYMBOLSTYLE_DOT
      DPSymbolType(2) = SYMBOLSTYLE_DOT
      DPSymbolType(3) = SYMBOLSTYLE_DOT
      DPLegend(0)     = char(0)
      DPLegend(1)     = char(0)
      DPLegend(2)     = char(0)
      DPLegend(3)     = char(0)
      DPTitle(1) = 'Air Speed/Ground Speed\n'   ! Add a linefeed to make room for labels at the top
      DPTitle(2)      = ' ' 
      DPTitle(3)      = ' ' 
      DPXAxis    = 'Wind Direction, degrees'
      DPYAxis    = 'Wind Speed/Ground Speed'

      ret = DPlot_Plot8(DPVersion,x,y,
     + '[Caption("DPLOTLIB Vector Plot Test")]'//
     + '[ManualScale(-10,-0.1,370,1.6)]'//
     + '[TickInterval(1,10,0.1)]'//
     + '[FontPoints(1,10)][FontPoints(6,10)][AxisType(2)]'//
     + '[XYLabel(0,1,0,"Tailwind")][XYLabel(90,1,0,"Left")]'//
     + '[XYLabel(180,1,0,"Headwind")][XYLabel(270,1,0,"Right")]'//
     + '[XYLabel(360,1,0,"Tailwind")]'//
     + '[VectorPlot(1,2,3,0x0000FF00,0x000000FF,100,500)]'//
     + '[VectorPlotOptions(6,150,20)]'//
     + '[DocMaximize()][ClearEditFlag()]'//char(0))

      return
      end
c
c=======================================================================
c
      subroutine MovieFrames()
      implicit none
      include  'dplot.g77'
c
c     Make movie frames of current 3D plot
c
c     NOTE: There's currently (2.2.4.1) no way to check for whether a 3D/4D plot is
c           displayed in a 3D view. This routine will work with 2D views w/o a problem, but
c           all frames will be identical.
c
      character szCmd*128
      character szValue*32
      integer   dwSize
      integer   az_increment
      integer   width, height
      integer   i, L, ret
      character szDestination*256
      integer   DocNum, DocType

      integer  lentrim
      external lentrim
      integer  DPlot_Command
      external DPlot_Command
      integer  DPlot_Request
      external DPlot_Request

c DPlot_Request bombs with g77. It doesn't seem to like the dwSize argument
c being passed by reference (which is correct!). No idea why.
c If you figure it out, please let me know at
c support@dplot.com

      dwSize = len(szValue)
      if(DPlot_Request(%VAL(0),'ActiveDocument'//char(0),szValue,dwSize)
     +               .lt. 0) then
        write(*,*) 'Error retrieving active document index from DP'
        return
      endif
      L = index(szValue,char(0))
      if(L .ne. 0) szValue(L:L)=' '
      read(szValue,*) DocNum
      if(DocNum .le. 0) then
        write(*,*) 'No document is open. To use this feature you '//
     +          'need an open 3D or 4D plot with a 3D view in DP'
        return
      endif

      dwSize = len(szValue)
      if(DPlot_Request(%VAL(DocNum),'DocumentType'//char(0),szValue,
     +      dwSize) .lt. 0) then
        write(*,*) 'Error retrieving document type (random 3D '//
     +             'points, etc.) from document index ',DocNum
        return
      endif
      L = index(szValue,char(0))
      if(L .ne. 0) szValue(L:L)=' '
      read(szValue,*) DocType
      if((DocType .ne. DATA_3D)  .and.    ! Z values on a rectangular grid
     +   (DocType .ne. DATA_3DR) .and.    ! Random X,Y,Z values
     +   (DocType .ne. DATA_3DS) .and.    ! 3D scatter plot
     +   (DocType .ne. DATA_4D ) .and.    ! 4D surface plot
     +   (DocType .ne. DATA_4DS)      ) then
        write(*,*) 'This feature only works with 3D/4D documents.'
        write(*,*) 'The currently active document is type ',DocType
        return
      endif
      write(*,*) 'Enter azimuth increment in degrees (0 to abort)... '
      read(*,*) az_increment
      if(az_increment .eq. 0) return
      write(*,*) 'Enter desired bitmap width and height, in pixels '//
     +           '(640x480 suggested)... '
      read(*,*) width, height
      write(*,*) 'Enter destination path for bitmap images... '
      read(*,'(A)') szDestination
      L = lentrim(szDestination)
      if(szDestination(L:L) .eq. '\') then
        szDestination(L:L) = ' '
        L = L-1
      endif

      write(szCmd,'(A,I4,1H,,I4,A,I4,1H,,I4,A,A)')
     +          '[SetClientRect(1,0,0,',width-1,height-1,
     +          ')][SetPluginImageDims(',width,height,
     +          ',24,0,0)][SetImageCrop(0)][Preserve3DScaleFactor()]',
     +          char(0)
      ret = DPlot_Command(DocNum,szCmd)
      do i=0,359,az_increment
        write(szCmd,'(A,I3,A)') '[ContourView(',i,')]'//char(0)
        ret = DPlot_Command(DocNum,szCmd)
        write(szCmd,'(A,A,1H\,I3.3,A)')
     +         '[FileSaveAs("BMP Picture","',
     +         szDestination(1:L),i,'.bmp")]'//char(0)
        ret = DPlot_Command(DocNum,szCmd)
        write(*,*) i
      end do
      ret = DPlot_Command(DocNum,'[Preserve3DScaleFactor(0)]'//char(0))
      return
      end
c
c=======================================================================
c
      subroutine Collatz()
      implicit none
      include  'dplot.g77'
c
c     Collatz Conjecture with transparent symbols.

      integer*4 NP
      parameter (NP=100000)
      integer*4 i, j, k
      integer*4 ret
      real*8    x(NP), y(NP)

      integer  DPlot_Plot8
      external DPlot_Plot8

      ret = 1

      do i=1,NP
        j = 0
        k = i
        do while(k .ne. 1)
           if(mod(k,2) .eq. 0) then
             k = k/2
           else
             k = k*3 + 1
           endif
           j = j+1
        end do
        x(i) = dble(i)
        y(i) = dble(j) 
      end do


      DPVersion       = DPLOT_DDE_VERSION
      DPhwnd          = 0
      DPDataFormat    = DATA_XYXY
      DPMaxCurves     = 1
      DPMaxPoints     = NP
      DPNumCurves     = 1
      DPScale         = SCALE_LINEARX_LINEARY
      DPNP(1)         = NP
      DPLineType(1)   = LINESTYLE_NONE
      DPSymbolType(1) = 288
      DPLegend(0)     = char(0)
      DPLegend(1)     = char(0)
      DPTitle(1)      = 'Collatz Conjecture' 
      DPTitle(2)      = ' ' 
      DPTitle(3)      = ' ' 
      DPYAxis         = 'Number of iterations'

      ret = DPlot_Plot8(DPVersion,x,y,
     +   '[Caption("DPLOTLIB Test")][DocMaximize()]'//
     +   '[ClearEditFlag()][SymbolSizeUnits(0)][SymbolSize(1,100)]'//
     +   '[SymbolOpacity(1,10)][FontPoints(7,10)]'//
     +   '[TextNoteEx("{\bCollatz Conjecture:}\n'//
     +   'Take any natural number {\in}. If {\in} is even, divide '//
     +   'it by 2\n'//
     +   'to get {\in}/2, if {\in} is odd multiply it by 3 and '//
     +   'add 1 to\n'//
     +   'obtain 3{\in} + 1. Repeat the process indefinitely. The\n'//
     +   'conjecture is that no matter what number you start with,\n'//
     +   'you will always eventually reach 1. This example plots\n'//
     +   'the number of iterations required to reach 1 for all\n'//
     +   'numbers between 1 and 100,000 (and also shows off\n'//
     +   'DPlot''s partially transparent symbols feature).",0,0,0,0)]'//
     +   char(0))
      return
      end
